//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.fulfillment.commands.*;

import java.sql.*;

import com.ibm.commerce.datatype.*;
import java.util.*;
import java.rmi.*;
import javax.ejb.*;
import javax.naming.*;

import com.ibm.commerce.base.objects.*;
import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.common.objects.*;

import com.ibm.commerce.ras.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.user.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.utils.*;

/**
 * Allocates expected inventory for a particular order item at a particular fulfillment center.
 * <p><strong>Behaviour</strong>
 * <p>The behaviour of the command is:
 * <ol><li> If the order item is suitable for backordering (i.e. its inventoryStatus attribute 
 * specifies unallocated), and there is sufficient expected inventory at one of the specified 
 * fulfillment centers (searching in order from first to last in the list of input fulfillment 
 * centers ), to completely backorder the quantity needed by the order item, backorder 
 * it and update the order item to reflect the backorder.
 *<li> Otherwise, the order item is not changed.
 *</ol>
 *<p>Note: in our default implementation, some expected inventory is represented by 
 * ReplenishmentAdvisments, and some expected inventory is represented by a StoreEntity 
 * policy allowing allocation from expected inventory at certain fulfillment centers even 
 * when there are no unallocated ReplenishmentAdvisements available (as if there was an 
 * expectation of infinite inventory for certain Stores at certain fulfillment centers), 
 * as long as the item is not discontinued.  For expected inventory represented by policy, 
 * the estimated availability time is derived from the policy, represented by the 
 * Store.defaultBackorderOffset attribute.
 *
 */
public class AllocateExpectedInventoryCmdImpl extends TaskCommandImpl implements AllocateExpectedInventoryCmd
{
	public static final String COPYRIGHT                 = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	protected static String istrThisClass                = "com.ibm.commerce.fulfillment.commands.AllocateExpectedInventoryCmdImpl";	
	protected static String istrPerformFunc              = "performExecute";
	protected static String istrCheckParametersFunc      = "checkParameters";

	// input parameters
	private Vector              ivFulfillmentCenters     = null;
	private OrderItemAccessBean iabOrderItem             = null;

	//private StoredProcedureHelperAccessBean 
	                            //iabStoredProcHelper      = new StoredProcedureHelperAccessBean();

	private String buildFFCList(Enumeration anenFulfillmentCenters)
	{
		StringBuffer strFFCList = new StringBuffer();
		boolean bSeparatorNeeded = false;
		while (anenFulfillmentCenters.hasMoreElements())
		{
			Integer nFulfillmentCenter = (Integer) anenFulfillmentCenters.nextElement();
			if (bSeparatorNeeded)
				strFFCList.append(InventoryManagementHelper.FFC_LIST_SEPARATOR);
			strFFCList.append(nFulfillmentCenter.toString());
			bSeparatorNeeded = true;
		}

		return strFFCList.toString();
	}
/**
 * Gets a prioritized list of fulfillment centers from which the inventory should be allocated.
 */
public java.util.Vector getFulfillmentCenters() {
	return ivFulfillmentCenters;
}
/**
 * Gets the order item for which expected inventory is to be allocated.
 *
 */
public com.ibm.commerce.order.objects.OrderItemAccessBean getOrderItem() {
	return iabOrderItem;
}
/**
 * Contains the actual business logic of the command.
 */
public void performExecute() throws ECException
{
	final String methodName = istrPerformFunc;
		
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);

//	checkParameters();

	try
	{	
		TypedProperty       tpSPResult    = null;
		OrderItemAccessBean abOrderItem   = getOrderItem();
		Long                nOrderItemId  = abOrderItem.getOrderItemIdInEJBType();

		// do nothing if an item is allocated, backordered or can not be backordered
		String strInventoryStatus = abOrderItem.getInventoryStatus();
		if (strInventoryStatus == null)
		{
			strInventoryStatus = InventoryManagementHelper.INV_NOT_ALLOCATED;
			abOrderItem.setInventoryStatus(strInventoryStatus);
			abOrderItem.commitCopyHelper  ();								
		}
		if (!strInventoryStatus.equals(InventoryManagementHelper.INV_NOT_ALLOCATED))
		{ // CH: to nl message
	      ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName, "Inventory status of order item " + nOrderItemId + " is '" + strInventoryStatus + "'." + "Unable to backorder an item with this status.");
		  return;
		}

		Enumeration   enFFCs        = getFulfillmentCenters().elements();
		if (enFFCs.hasMoreElements())
  	    {
	  	    // String strFFCList = buildFFCList(enFFCs);
	  	    String strFFCList = InventoryManagementHelper.ffcsToString(enFFCs);
	  	    ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName, "Backordering item " + abOrderItem.getOrderItemId() + " from fulfillment centers " + strFFCList);

	  	    // backorder an order item from a list of ffcs
			tpSPResult = StoredProcedureHelperProxy.callBOItem(nOrderItemId, strFFCList);
						
			String  strStatus = tpSPResult.getString(InventoryManagementHelper.STATUS);		
		
			if (strStatus.trim().equals(InventoryManagementHelper.STATUS_SUCCESS))
			{   // allocation is successful
				Integer nFulfillmentCenter = tpSPResult.getInteger(InventoryManagementHelper.FUFILLMENT_CENTER);
				String  strEstShipDate     = tpSPResult.getString (InventoryManagementHelper.EST_SHIP_DATE    );

				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName, "Backordered order item " + nOrderItemId + " from fulfillment center " + nFulfillmentCenter + " with expected shipping date " + strEstShipDate);

				InventoryManagementHelper.setOrderItemAllocationInfo(abOrderItem, nFulfillmentCenter, abOrderItem.getNeededQuantityInEJBType(), InventoryManagementHelper.INV_BACKORDERED, strEstShipDate);
			
			}
			else
			{   // item is discontinued, marked for deletion or not backorderable				
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName, "Could not backorder item " + nOrderItemId + "; status : " + strStatus);
			}			
  	    }		

	}
	catch (RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}
	catch (SQLException    e) { throw new ECSystemException(ECMessage._ERR_SQL_EXCEPTION   , istrThisClass, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e );}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, methodName);
}
/**
 * Sets a prioritized list of fulfillment centers from which the inventory should be allocated.
 */
public void setFulfillmentCenters(java.util.Vector newFulfillmentCenters) {
	ivFulfillmentCenters = newFulfillmentCenters;
}
/**
 * Sets the order item for which expected inventory is to be allocated.
 *
 */
public void setOrderItem(com.ibm.commerce.order.objects.OrderItemAccessBean newOrderItem) {
	iabOrderItem = newOrderItem;
}
/**
 * Performs server side parameter checking.
 */
public void validateParameters() throws com.ibm.commerce.exception.ECException {
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, istrCheckParametersFunc);

	if (getOrderItem() == null)
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, istrThisClass, istrCheckParametersFunc, ECMessageHelper.generateMsgParms(OrderConstants.EC_ORDER_ID));

	if (getFulfillmentCenters() == null)
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, istrThisClass, istrCheckParametersFunc, ECMessageHelper.generateMsgParms(OrderConstants.EC_FULFILLMENT_CENTER_ID));

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, istrThisClass, istrCheckParametersFunc);
}
}
