//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.negotiation.commands.*;

import javax.ejb.*;
import java.rmi.*;
import javax.naming.*;

import com.ibm.commerce.command.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.sfc.cmd.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.negotiation.objects.*;
import com.ibm.commerce.negotiation.util.*;
import com.ibm.commerce.negotiation.misc.*;

/**
 * Determines if a bidder is allowed to Delete a bid.
 * There are several reasons to disallow bid Delete operation:
 * <UL>
 * <LI> The bidder is not authorized to Delete the bid
 * <LI> The auction rule forbid a Delete operation
 * <LI> The bid being updated is the best bid
 * </UL>
 * The current implementation will ONLY check whether the bidder is
 * authorized to Delete the bid.
 */
public class CheckBidDeleteAuthorizationCmdImpl
	extends TaskCommandImpl
	implements CheckBidDeleteAuthorizationCmd, AuctionConstants

{
	public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;


	protected boolean bidderAuthorized = false;
	private java.lang.String bidId = null;
/**
 * CheckBidDeleteAuthorizationImpl default constructor.
 */
public CheckBidDeleteAuthorizationCmdImpl()
{
	super();
}
/**
 * Gets the bid identifier field.
 * @return The bid identifier.
 */
public java.lang.String getBidId()
{
	return bidId;
}
/**
 * Gets the bidder authorized status.
 * @return true if the bidder is authorized, false otherwise.
 */
public boolean isBidderAuthorized()
{
	return bidderAuthorized;
}
/**
 * Verifies that the this command is ready to be executed.
 * @return true if command is ready, otherwise false.
 */
public boolean isReadyToCallExecute()
{
	final String methodName = "isReadyToCallExecute";

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName);

	if (!super.isReadyToCallExecute())
		return false;

	if (getBidId() == null)
	{
		return false;
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName);
	return true;
}
/**
 * Determine if current bidder is authorized to Delete the bid specified
 * by the bid ID.
 * @exception ECException if any problems are encountered.
 */
public void performExecute() throws ECException
{
	final String methodName = "performExecute";

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName );

	setBidderAuthorized(false);

	Long currUserNumber = getUserId();
	if (currUserNumber == null)
	{
		throw new ECApplicationException( ECMessage._ERR_DIDNT_SET_SHOPPER,
		                                  this.getClass().getName(),
		                                  methodName );
	}

	String userKey = currUserNumber.toString();

	try
	{
		// Lookup the bid
		BidAccessBean bab = new BidAccessBean();
		bab.setInitKey_id(Long.valueOf(getBidId()));
		bab.refreshCopyHelper();

		// Can only delete active bids, verify this one is active
		if (bab.getStatus().equals(EC_BID_STATUS_ACTIVE))
		{
			// The bid is active, but only its owner may delete it.
			// Verify that the user owns the bid.
			if (bab.getOwnerId().equals( userKey ))
			{
				// This person is authorized to delete the bid.
				setBidderAuthorized(true);
			}
			else
			{
				// User does not own the bid!
				throw new ECApplicationException(ECMessage._ERR_BID_NOT_OWNED,
												this.getClass().getName(),
												methodName,
												ECMessageHelper.generateMsgParms(getBidId(), userKey));
			}
		}
		else
		{
			// This bid is not active!
			throw new ECApplicationException(ECMessage._ERR_BID_NOT_ACTIVE,
				                             this.getClass().getName(),
				                             methodName,
				                             ECMessageHelper.generateMsgParms(getBidId(), bab.getStatus()));
		}
	}
	catch (CreateException e)
	{
		throw new ECApplicationException(ECMessage._ERR_CREATE_EXCEPTION,
			                        this.getClass().getName(),
			                        methodName,
			                        ECMessageHelper.generateMsgParms(e.getMessage()));
	}
	catch (NumberFormatException e)
	{
		throw new ECApplicationException(ECMessage._ERR_NUMBER_FORMAT_EXCEPTION,
			                        this.getClass().getName(),
			                        methodName,
			                        ECMessageHelper.generateMsgParms(e.getMessage()));
	}
	catch (RemoteException e)
	{
		throw new ECApplicationException(ECMessage._ERR_REMOTE_EXCEPTION,
			                        this.getClass().getName(),
			                        methodName,
			                        ECMessageHelper.generateMsgParms(e.getMessage()));
	}
	catch (FinderException e)
	{
		throw new ECApplicationException(ECMessage._ERR_FINDER_EXCEPTION,
			                        this.getClass().getName(),
			                        methodName,
			                        ECMessageHelper.generateMsgParms(e.getMessage()));
	}
	catch (NamingException e)
	{
		throw new ECApplicationException(AuctionMessage._ERR_NAMING_EXCEPTION,
			                        this.getClass().getName(),
			                        methodName,
			                        ECMessageHelper.generateMsgParms(e.getMessage()));
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_NEGOTIATION, this.getClass().getName(), methodName );
}
/**
 * Sets the bidder authorized status.
 * @param newBidderAuthorized The bidder's authorization status.
 */
protected void setBidderAuthorized(boolean newBidderAuthorized)
{
	bidderAuthorized = newBidderAuthorized;
}
/**
 * Sets the bid identifier field.
 * @param newBidId The bid identifier.
 */
public void setBidId(java.lang.String newBidId)
{
	bidId = newBidId;
}
}
