//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.price.commands.*;

import com.ibm.commerce.catalog.objects.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.objimpl.*;
import java.util.*;
import java.rmi.*;
import java.sql.*;
import java.math.*;

import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.price.utils.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.common.objimpl.*;

/**
 * Get the best price for a catalog entry.
 * <p>The offer that is used to find the price of a catalog entry Id must satisfy the following
 * criteria:
 * <ul>
 * <li>offer Id matches one of the supplied offer Ids or no offer Ids were supplied.
 * <li>the offer must be associated with a trading position container that either has no
 * member groups associated with it or has a member group associated with it that the shopper
 * is explicitly a member of (MBRGRPMBR table).
 * <li>the trading position container must be associated with a contract that is associated
 * with the same store under which the command is running.
 * <li>the offer must be published.
 * <li>the current timestamp must be in the active range specified for the offer (between the
 * STARTDATE and ENDDATE).
 * <li>the offer will have the highest possible precedence within its trading position container.
 * A lower precedence offer may be chosen if there is no qualifying offer with a higher precendence
 * level in the same trading position container.
 * <li>at most one offer will be chosen from each trading position container. The best of each offer
 * from each trading position container will be the final offer.
 * <li>either no quantityAmount was specified (see <code>setQuantity(QuantityAmount)</code> or the quantity range
 * specified on the offer includes the quantity amount specified.  If a conversion is required and
 * cannot be performed (because there is no conversion rule) then the offer is not considered.
 * <li>there is an offer price for the offer in the correct currency or a conversion exists from
 * the offer price currency to the requested currency.  If there is more than one currency that can
 * be converted to the requested currency then the lowest price will be used from all the conversions.
 * </ul>
 * <p>If a valid offer cannot be found for an item, the parent product offers will be searched.
 */

public class GetBaseUnitPriceCmdImpl extends com.ibm.commerce.command.TaskCommandImpl implements GetBaseUnitPriceCmd, GetBaseSpecialPriceCmd, GetProductBaseUnitPriceCmd {
	private static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;	
	private Long iCatEntryId = null;

	private MonetaryAmount iPrice = null;
	private MonetaryAmount iBasePrice = null;
	private Timestamp iExpiryDateForCache = null;
		
	private Long iOfferId = null;
	private Long[] iContractIdsIn = null;
	private Long[] iMemberGroupIds = null;
	private OrderItemAccessBean iOrderItemAccessBean = null;
	private final static java.lang.String iClassName = GetBaseUnitPriceCmdImpl.class.getName();
	private com.ibm.commerce.price.utils.QuantityAmount iQuantityAmount = null;
	private com.ibm.commerce.order.objects.OfferAccessBean iOffer = null;
	private Long[] iOfferIds = null;
	private java.lang.String iCurrency = null;
	private com.ibm.commerce.fulfillment.objects.CatalogEntryShippingAccessBean iCatalogEntryShippingAB = null;
	private java.lang.Integer iStoreId = null;
	private com.ibm.commerce.common.objects.StoreAccessBean iStoreAB = null;
	private Long[] iContractIdsOut = null;
	private final static java.math.BigDecimal BIG_DECIMAL_ZERO = new BigDecimal("0");
	private final static Timestamp BIG_TIMESTAMP = Timestamp.valueOf("9999-12-30 23:59:59.000");;
	private class  QualifyingOfferInfo {
		private OfferAccessBean offer = null;
		private BigDecimal lowestPriceValue = null;
		private MonetaryAmount lowestBasePrice = null;
		QualifyingOfferInfo() {}
		public void setInfo(OfferAccessBean anOffer, BigDecimal aLowestPriceValue, MonetaryAmount aLowestBasePrice) {
			offer = anOffer;
			lowestPriceValue = aLowestPriceValue;
			lowestBasePrice = aLowestBasePrice;
		}
		public OfferAccessBean getOffer() { return offer; }
		public BigDecimal getLowestPriceValue() { return lowestPriceValue; }
		public MonetaryAmount getLowestBasePrice() { return lowestBasePrice; }
		public void reset() {
			offer = null;
			lowestPriceValue = null;
			lowestBasePrice = null;
		}
	}	
	private java.sql.Timestamp priceTimestamp = null;
/**
 * GetBaseUnitPriceCmdImpl constructor.
 */
public GetBaseUnitPriceCmdImpl() {
	super();
}
/**
 * Insert the method's description here.
 * @return com.ibm.commerce.price.utils.QuantityAmount
 * @param aQuantity double
 */
private QuantityAmount calculateQuantityAmount(double aQuantity) throws ECSystemException {
	QuantityAmount quantityAmount;
	
	if( iCatalogEntryShippingAB == null ) {
		iCatalogEntryShippingAB = Helper.getCatalogEntryShippingAB(getCatEntryId());
	}

	quantityAmount = Helper.createQuantityAmount(iCatalogEntryShippingAB, aQuantity);

	return quantityAmount;
}
/**
 * Check to see if this is a cache target and, if so, set the timestamp appropriately.
 * Creation date: (9/27/00 12:37:35 PM)
 */
private void checkCaching() throws ECSystemException {
	final String methodName = "checkCaching";

	// check to see if we need to supply a cache expiry date
	if (commandContext.isCacheTarget()) {
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "Cache Target = true");
		Timestamp thisExpiryDate = getExpiryDateForCache();

		// if the expiry date of this offer is set then
		if( thisExpiryDate != null ) {

			Timestamp oldExpiryDate = commandContext.getExpiryDate();
			// do we have an earlier expiry date
			if ((oldExpiryDate == null) || thisExpiryDate.before(oldExpiryDate)) {
				// yes - so replace the later one in the command context
				commandContext.setExpiryDate(thisExpiryDate);
			}
		}
	}
}
/**
 * Execute the command.
 */
private QualifyingOfferInfo findBestOffer(Long aCatEntryId, String aRequestedCurrency, StoreGetPriceInputData theInputData) throws ECException {
	final String methodName = "findBestOffer";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
	
	Vector offerAccessBeanVector;
	QualifyingOfferInfo globalOfferInfo = null;

	try {
		Integer storeId = getStoreAB().getStoreEntityIdInEJBType();
				
		// get all offers that qualify
		offerAccessBeanVector = getStoreAB().getAvailableMaxPrecedenceOffers(aCatEntryId, theInputData );

		QualifyingOfferInfo currentOfferInfo = new QualifyingOfferInfo();
		
		// if there are offers
		if (offerAccessBeanVector != null) {	

			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "number of returned offers = " + offerAccessBeanVector.size());

			// these variables keep track of info about the last qualifying offer found
			Long tradingPosContainerId = new Long(Long.MIN_VALUE);	// trading position container of last found qualifying offer
			Double currentPrecendence = new Double(Double.MIN_VALUE);	// precedence of last qualifying offer
						
			// iterate over all offers	
			for (int i = 0; i < offerAccessBeanVector.size(); i++) {
				OfferAccessBean offer = (OfferAccessBean) offerAccessBeanVector.elementAt(i);

				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "processing offerID = " + offer.getOfferIdInEJBType());
				
				// have we found a current offer
				if( null != currentOfferInfo.getOffer() ) {

					// if we are entering a new container then 
					if( !tradingPosContainerId.equals(offer.getTradePosContainerIdInEJBType()) ) {
						// store to global info if the price is better
						if( globalOfferInfo == null ||
							isBestPrice( globalOfferInfo, currentOfferInfo.getLowestPriceValue() ) ) {
								globalOfferInfo = currentOfferInfo;
								currentOfferInfo = new QualifyingOfferInfo();
							}
					} else {
						// we are still in the same container
						// are we entering a new precedence?
						if (!currentPrecendence.equals(offer.getPrecedenceInEJBType()) ) {
							// skip this offer
							continue;
						}
					}
				}
				
				// Do I qualify for this offer
				if( !isValidOffer( offer ) ) continue;
			
				// priceOffer.getPrices() will either return 1 price if the requested currency has a price OR
				// 1 or more prices if the requested currency does not have an explicit price.
				Vector offerPricesVector = offer.getPrices(aRequestedCurrency);

				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "-- number of prices per offer = " + offerPricesVector.size());
				
				boolean isOfferProcessed = false;
				// Process the price(s) associated with the offer
				// if there is only one price for an offer and that price is in the correct currency, use that price
				if (offerPricesVector.size() == 1) {
					OfferPriceAccessBean offerPrice = (OfferPriceAccessBean) offerPricesVector.firstElement();
					if (offerPrice.getCurrency().equals(aRequestedCurrency)) {

						// is the new price better than a previous offer
						BigDecimal newPriceValue = offerPrice.getPriceInEJBType();
						if( isBestPrice( currentOfferInfo, newPriceValue ) ) {
							currentOfferInfo.setInfo(offer, newPriceValue, null);
							currentPrecendence = offer.getPrecedenceInEJBType();
							tradingPosContainerId = offer.getTradePosContainerIdInEJBType();
						}

						// offer price has already been processed
						isOfferProcessed = true;
					}
				}

				// if we haven't already processed the offer
				if (!isOfferProcessed) {
					// convert all the prices to the target currency and choose the lowest
					if (processOfferPricesVector(currentOfferInfo, offerPricesVector, aRequestedCurrency, offer)) {
						// there was a successful conversion to a better price
						currentPrecendence = offer.getPrecedenceInEJBType();
						tradingPosContainerId = offer.getTradePosContainerIdInEJBType();
					}
				}
			}
		}

		if( currentOfferInfo.getOffer() != null ) {
			if( globalOfferInfo == null ||
				isBestPrice( globalOfferInfo, currentOfferInfo.getLowestPriceValue() ) ) {
				globalOfferInfo = currentOfferInfo;
			}
		}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

	return globalOfferInfo;
}
/**
 * Gets the base price of the catalog entry.
 * The base price is the price (and currency) 
 * found for the catalog entry before any conversion.  This is only set
 * when the requested currency is different from the base currency
 * and the base price must be converted to the requested price.
 * @return the base price of the product.
 */
public MonetaryAmount getBasePrice() {
	return iBasePrice;
}
/**
 * Insert the method's description here.
 * Creation date: (4/18/00 8:53:34 PM)
 * @return java.lang.Integer
 */
private Long getCatEntryId() {
	return iCatEntryId;
}
/**
 * Get the contract IDs associated with the offer that supplied the price.
 * @return the contract ID(s).
 */
public Long[] getContractIds() throws ECSystemException {
	final String methodName = "getContractIds";

	/*
	try {
		if (iContractIdsOut == null) {
			
			if (iOffer != null) {
				Enumeration contractsEnum = iOffer.getContracts(myGetStoreId());
				Vector contractsVector = new Vector();
				while (contractsEnum.hasMoreElements()) {
					ContractAccessBean contractAB = (ContractAccessBean) contractsEnum.nextElement();
					contractsVector.addElement(contractAB.getContractIdInEJBType());
				}
				int size = contractsVector.size();
				if (size > 0) {
					iContractIdsOut = new Long[size];
					for (int i = 0; i < size; i++) {
						iContractIdsOut[i] = (Long) contractsVector.elementAt(i);
					}
				}
			}
			
		}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	}
	*/

	//
	return iContractIdsOut;
}
/**
 * Get the currency to use for the price.
 * @return java.lang.String
 */
private java.lang.String getCurrency() throws ECSystemException {
	if (iCurrency == null) {
		CommandContext context = getCommandContext();

		//
		iCurrency = Helper.getCurrency( context, getStoreAB() );
	}
	return iCurrency;
}
/**
 * Get the expiry date of the price.  The expiry date of the price may be influenced by any other
 * offer even those that the user does not qualify for.  This simplifies the algorithm that decides
 * which page to display for a particular user and also simplifies the algorithm for cache expiration.
 * @return the expiry date. 
 */
public java.sql.Timestamp getExpiryDateForCache() throws ECSystemException {
	final String methodName = "getExpiryDateForCache";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
	try {
		if (iExpiryDateForCache == null) {
			Timestamp endDate = iOffer.getEndDateInEJBType();
			Timestamp endDateToSearch = endDate;
			if( endDateToSearch == null ) endDateToSearch = BIG_TIMESTAMP;
//			Timestamp startDate = iOffer.getStartDateInEJBType();

			// get the starting date of the next offer
			OfferAccessBean offerAB = new OfferAccessBean();
			OfferAccessBean possibleOfferAB = null;
			try {
				possibleOfferAB = offerAB.findByTimestampAndStore(priceTimestamp, endDateToSearch, myGetStoreId());
			} catch (javax.ejb.ObjectNotFoundException ex) {
				// do nothing
			}

			// if there is another offer that has a starting date before this offer's ending date
			// then use that as the expiry date.
			if (possibleOfferAB != null) {
				iExpiryDateForCache = possibleOfferAB.getStartDateInEJBType();
			} else {
				//use the offer's ending date as the expiry date
				iExpiryDateForCache = endDate;
			}
		}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	}

	//
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
	return iExpiryDateForCache;
}
/**
 * Insert the method's description here.
 * @return java.lang.Long[]
 */
private Long[] getMemberGroupIds() throws ECSystemException {
	final String methodName = "getMemberGroups";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
	try {
		if (iMemberGroupIds == null) {
			UserAccessBean userAB = commandContext.getUser();

			//@@D30756 - don't get member groups if this is a generic shopper
			if (!userAB.getUserId().equals(ECConstants.EC_GENERIC_USER_REFNUM)) {
				Integer storeId = getStoreAB().getStoreEntityIdInEJBType();
				//			MemberGroupMemberAccessBean memberGroupsAB = new MemberGroupMemberAccessBean();
				//			Enumeration memberGroupsEnum = memberGroupsAB.findAllStoreGroupMember(getStoreAB().getStoreEntityIdInEJBType(), commandContext.getUserId());
				Vector memberGroupsVector = userAB.getAllStoreGroupMembers(storeId);

				ECTrace.trace(
					ECTraceIdentifiers.COMPONENT_ORDER,
					iClassName,
					methodName,
					"userId = " + userAB.getUserId() + ", storeId = " + storeId);

				// create an array from vector
				int size = memberGroupsVector.size();
				if (size > 0) {
					iMemberGroupIds = new Long[size];

					// create array
					for (int i = 0; i < size; i++) {
						MemberGroupMemberAccessBean memberGroup = (MemberGroupMemberAccessBean) memberGroupsVector.elementAt(i);
						iMemberGroupIds[i] = memberGroup.getMbrGrpIdInEJBType();
						ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "memberGroupId = " + iMemberGroupIds[i]);
					}
				}
			}
		}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	}
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

	return iMemberGroupIds;
}
/**
 * Get the offer access bean that was used to determine the price.
 * @return the offer access bean associated with the offer used to calculate the price.
 */
public OfferAccessBean getOffer() {
	return iOffer;
}
/**
 * getOfferID method comment.
 * @return the offer ID associated with the price.
 */
public Long getOfferId() {
	return iOfferId;
}
/**
 * Insert the method's description here.
 * @return java.lang.Long[]
 */
private java.lang.Long[] getOfferIds() {
	return iOfferIds;
}
/**
 * Insert the method's description here.
 * @return java.lang.Integer
 */
private OrderItemAccessBean getOrderItemAccessBean() {
	return iOrderItemAccessBean;
}
/**
 * Insert the method's description here.
 * Creation date: (12/7/00 10:37:16 AM)
 * @return java.util.Enumeration
 */
private Enumeration getParentCatEntryIds(Long catEntryId) throws ECSystemException {
	final String methodName = "getParentCatEntryIds";

	Enumeration parentProductEnum = null;
	try {
		CatalogEntryAccessBean catEntryAB = new CatalogEntryAccessBean();
		parentProductEnum = catEntryAB.findByChildCatalogEntry(catEntryId);

	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	}

	return parentProductEnum;
}
/**
 * Gets the actual price of the catalog entry.
 * This price is in the requested currency of the shopper.
 * @return the price of the catalog entry Id.
 */
public MonetaryAmount getPrice() {
	return iPrice;
}
/**
 * Insert the method's description here.
 * @return com.ibm.commerce.price.utils.QuantityAmount
 */
private com.ibm.commerce.price.utils.QuantityAmount getQuantity() throws ECSystemException {
	// if the quantity was not supplied, use the rounded quantity from the catalog entry shipping record
	if (iQuantityAmount == null) {
		iQuantityAmount = getRoundedQuantityAmount(1.0);
	}

	//
	return iQuantityAmount;
}
/**
 * Insert the method's description here.
 * @return com.ibm.commerce.price.utils.QuantityAmount
 * @param aQuantity double
 */
private QuantityAmount getRoundedQuantityAmount(double aQuantity) throws ECSystemException {
	QuantityAmount quantityAmount;
	
	if( iCatalogEntryShippingAB == null ) {
		iCatalogEntryShippingAB = Helper.getCatalogEntryShippingAB(getCatEntryId());
	}

	quantityAmount = Helper.createRoundedQuantityAmount(iCatalogEntryShippingAB, aQuantity);

	return quantityAmount;
}
/**
 * Insert the method's description here.
 * @return com.ibm.commerce.common.objects.StoreAccessBean
 */
private com.ibm.commerce.common.objects.StoreAccessBean getStoreAB() {
	if( iStoreAB == null ) {
		Integer storeId = myGetStoreId();
		if( storeId != null ) {
			setStoreAB(WcsApp.storeRegistry.find(storeId));
		}
	}
	return iStoreAB;
}
/**
 * Insert the method's description here.
 * @return boolean
 * @param newPriceValue BigDecimal
 */
private boolean isBestPrice(QualifyingOfferInfo offerInfo, BigDecimal aPriceValue) {
	BigDecimal lowestPriceValue = offerInfo.getLowestPriceValue();
	if ((lowestPriceValue == null) || (aPriceValue.compareTo(lowestPriceValue) < 0))
		return true;
	else
		return false;
}
/**
 * Do I qualify for this offer?
 * @return whether I qualify for this offer.
 * @param anOffer an offer access bean.
 */
private boolean isValidOffer(OfferAccessBean anOffer) throws ECException {
    final String methodName = "isValidOffer";
    ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

    boolean isValid = true;

    try {
        // minor workaround - need a new finder method for a particular offer 
        if (iOfferId != null) {
            isValid = iOfferId.equals(anOffer.getOfferIdInEJBType());
        }

        double quantityValue = getQuantityValue();
        boolean conversionValid = true;
        if (isValid) {
            double minimumQuantity;
            double maximumQuantity;

            Double minQuantity = anOffer.getMinimumQuantityInEJBType();
            Double maxQuantity = anOffer.getMaximumQuantityInEJBType();

            // process minimum amount	
            if (minQuantity == null) {
                minimumQuantity = 0.0;
            } else {
                minimumQuantity = minQuantity.doubleValue();
            }

            // process maximum amount
            if (maxQuantity == null) {
                maximumQuantity = Float.POSITIVE_INFINITY;
            } else {
                maximumQuantity = maxQuantity.doubleValue();
            }

            String qtyUnit = anOffer.getQuantityUnitId();
            // optimization: If offer.quantityUnit = quantity unit of requested quantity,
            // don't do conversions to catalog entry shipping record.
            // This line effectively only checks that the 2 quantities are null.  The chances of both 
            // quantity values pointing to the same memory location are slim.  If the 2 quantity 
            // values are equal() and not null, the path in the else portion will be short and will 
            // avoid the extra SQL for CATENTSHIP.
            if (qtyUnit == getQuantityUnit()) {
                // don't do conversions
            } else {
                if (qtyUnit == null) {
                    // convert maximum and minimum to nominal quantity in the catalog entry
                    QuantityAmount minQuantityAmount = calculateQuantityAmount(minimumQuantity);
                    QuantityAmount maxQuantityAmount = calculateQuantityAmount(maximumQuantity);

                    // set values to new calculated amounts
                    qtyUnit = minQuantityAmount.getQuantityUnit();
                    minimumQuantity = minQuantityAmount.getValue();
                    maximumQuantity = maxQuantityAmount.getValue();
                }

                // convert to the expected quantity unit
                // get the quantity manager
                QuantityManager quantityManager = QuantityManager.getInstance();

                // get converted quantity amount
                QuantityAmount convertedQuantityAmount =
                    quantityManager.convert(getQuantity(), qtyUnit);

                if( convertedQuantityAmount == null ) {
	                conversionValid = false;
                } else {
                	quantityValue = convertedQuantityAmount.getValue();
                }
            }

            // if no valid conversion or the quantity does not qualify for this offer
            if (!conversionValid
                || (minimumQuantity > quantityValue)
                || (maximumQuantity < quantityValue)) {
                isValid = false;
            }
        }
    } catch (javax.ejb.CreateException ex) {
        throw new ECSystemException(
            ECMessage._ERR_CREATE_EXCEPTION,
            iClassName,
            methodName,
            ex);
    } catch (javax.ejb.FinderException ex) {
        throw new ECSystemException(
            ECMessage._ERR_FINDER_EXCEPTION,
            iClassName,
            methodName,
            ex);
    } catch (javax.naming.NamingException ex) {
        throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
    } catch (java.rmi.RemoteException ex) {
        throw new ECSystemException(
            ECMessage._ERR_REMOTE_EXCEPTION,
            iClassName,
            methodName,
            ex);
    }

    ECTrace.trace(
        ECTraceIdentifiers.COMPONENT_ORDER,
        iClassName,
        methodName,
        "valid offer = " + isValid);
    ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

    return isValid;
}
/**
 * Insert the method's description here.
 * @return java.lang.Integer
 */
private java.lang.Integer myGetStoreId() {
	if( iStoreId == null ) iStoreId = getStoreId();
	return iStoreId;
}
/**
 * Execute the command.
 */
public void performExecute() throws ECException {
	final String methodName = "performExecute";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

	// check that all parameters are supplied
//	checkParameters();
	
	String requestedCurrency;
	QualifyingOfferInfo globalOfferInfo = null;

	try {
		CommandContext commandContext = getCommandContext();
		requestedCurrency = getCurrency();

		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "currency used for prices = " + requestedCurrency);

		Integer storeId = getStoreAB().getStoreEntityIdInEJBType();
		Long catEntryId = getCatEntryId();
				
		// get all offers that qualify
		StoreGetPriceInputData inputData = new StoreGetPriceInputData( getOfferIds(), iContractIdsIn, getMemberGroupIds() );

		if (commandContext.isCacheTarget()) {
//			com.ibm.commerce.cache.objects.CacheLogAccessBean cacheLogAB = new com.ibm.commerce.cache.objects.CacheLogAccessBean();
//			priceTimestamp = cacheLogAB.findDatabaseCurrentTimestamp();
			priceTimestamp = new com.ibm.commerce.base.objects.ServerJDBCHelperAccessBean().getCurrentTimestamp();
		}
		
		globalOfferInfo = findBestOffer(catEntryId, requestedCurrency, inputData);

		// can't find a price for the item?
		if( globalOfferInfo == null ) {
			// get the parent products
			Enumeration parentProductsEnum = getParentCatEntryIds(catEntryId);

			QualifyingOfferInfo lOfferInfo;
			while( parentProductsEnum.hasMoreElements() ) {
				Long productId = ((CatalogEntryAccessBean) parentProductsEnum.nextElement()).getCatalogEntryReferenceNumberInEJBType();

				lOfferInfo = findBestOffer(productId, requestedCurrency, inputData);

				// is this the best offer of the parents?
				if( (globalOfferInfo == null) || isBestPrice( globalOfferInfo, lOfferInfo.getLowestPriceValue() ) ) {
					globalOfferInfo = lOfferInfo;
				}
			}
		}

		OfferAccessBean lowestOffer = ((globalOfferInfo == null)? null : globalOfferInfo.getOffer());
		// all processing is complete, set up the results
		if( lowestOffer != null ) {
			setOffer(lowestOffer);
			setOfferId( lowestOffer.getOfferIdInEJBType() );
			setBasePrice(globalOfferInfo.getLowestBasePrice());
			setPrice( new MonetaryAmount( globalOfferInfo.getLowestPriceValue(), requestedCurrency ));

			// Caching Code
			checkCaching();
		} else {
			// raise an exception
			com.ibm.commerce.datatype.TypedProperty nvps = new com.ibm.commerce.datatype.TypedProperty();
			nvps.put( ECConstants.EC_ERROR_CODE, GetBaseUnitPriceCmd.ERR_CODE_NO_PRICE );
			nvps.put( OrderConstants.EC_CATALOG_ENTRY_ID, getCatEntryId().toString() );
			nvps.put( OrderConstants.EC_QUANTITY, Double.toString(getQuantity().getValue()));
			nvps.put( OrderConstants.EC_UOM, getQuantity().getQuantityUnit() );
			nvps.put( ECConstants.EC_CURRENCY_ID, requestedCurrency );
			Object[] msgParams = { getCatEntryId().toString(), 
									Double.toString(getQuantity().getValue()),
									getQuantity().getQuantityUnit(),
									requestedCurrency };
			ECApplicationException exception = new ECApplicationException( ECMessage._ERR_RETRIEVE_PRICE, iClassName, methodName, msgParams, GetBaseUnitPriceCmd.ERRTASK_NAME, nvps );
			exception.setOrginatingCommand(iClassName.substring(1+iClassName.lastIndexOf('.')));
			throw exception;
		}

	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	} catch (java.sql.SQLException ex) {
		throw new ECSystemException(ECMessage._ERR_SQL_EXCEPTION, iClassName, methodName, ex);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
}
/**
 * Process the prices for a specific offer.  It is assumed that there is no price for
 * the shopper's requested currency and therefore all prices are converted to the
 * requested currency and the lowest one is chosen.
 * @param offerPricesVector a vector of prices for an offer
 * @param requestedCurrency the shopper's requested currency
 * @return whether there was a successful conversion from any of the prices to the requested currency (and its the best yet).
 */
private boolean processOfferPricesVector(QualifyingOfferInfo offerInfo, Vector offerPricesVector, String requestedCurrency, OfferAccessBean offer ) throws ECException {
	final String methodName = "processOfferPricesVector";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

	boolean isSuccessful = false;
	try {
		CurrencyManager currencyManager = CurrencyManager.getInstance();

		// convert each price to the requested currency
		MonetaryAmount[] existingPrices = new MonetaryAmount[offerPricesVector.size()];
		for (int i = 0; i < offerPricesVector.size(); i++) {
			OfferPriceAccessBean offerPrice = (OfferPriceAccessBean) offerPricesVector.elementAt(i);
			existingPrices[i] = new MonetaryAmount(offerPrice.getPriceInEJBType(), offerPrice.getCurrency());
		}

		PriceConversionAlgorithm priceConversion = new PriceConversionAlgorithm(existingPrices, requestedCurrency, getStoreAB());
		boolean priceFound = priceConversion.calculateLowestPrice();

		// did we get a successful conversion for this offer
		if (priceFound) {
				
			MonetaryAmount lowestPrice = priceConversion.getBestPrice();
			
			// set instance lowest information
			if( isBestPrice(offerInfo, lowestPrice.getValue()) ) {
				offerInfo.setInfo(offer, lowestPrice.getValue(), priceConversion.getBasePrice());
				isSuccessful = true;
			}
		}
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, iClassName, methodName, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, iClassName, methodName, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, iClassName, methodName, ex);
	}
	
	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName, "successfully converted = " + isSuccessful);
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
	
	return isSuccessful;
}
/**
 * reset the command.
 */
public void reset() {
	iBasePrice = null;
	iCatEntryId = null;
	iCatalogEntryShippingAB = null;
	iContractIdsIn = null;
	iContractIdsOut = null;
	iCurrency = null;
	iExpiryDateForCache = null;
	iMemberGroupIds = null;
	iOffer = null;
	iOfferId = null;
	iOfferIds = null;
	iOrderItemAccessBean = null;
	iPrice = null;
	iQuantityAmount = null;
	iStoreId = null;
	iStoreAB = null;
}
/**
 * Insert the method's description here.
 * Creation date: (10/4/99 10:00:54 PM)
 * @param newBasePrice java.lang.String
 */
private void setBasePrice(MonetaryAmount aBasePrice) {
	iBasePrice = aBasePrice;
}
/**
 * Sets the primary key of the catalog entry whose price is to be retrieved.
 * @param aCatEntryId the reference number of a catalog entry.
 */
public void setCatEntryId(Long aCatEntryId) {
	iCatEntryId = aCatEntryId;
}
/**
 * (Optional) Sets the contract IDs to be used to determine which offers apply.
 * If no contract ID is specified then the default contract for the store is used.
 * @param aContractIdList the contract IDs to use.
 */
public void setContractIds(Long[] aContractIdList) {
	iContractIdsIn = aContractIdList;
}
/**
 * (Optional) Set the currency to be used for finding the price.  If the currency is not
 * specified then the "negotiated" currency is used for the store Id and shopper's
 * preferred currency.  (The negotiated currency is the currency returned from the
 * getCurrency() method of the currency manager).
 * @param aCurrency the currency.
 */
public void setCurrency(String aCurrency) {
	iCurrency = aCurrency;
}
/**
 * Insert the method's description here.
 * @param newOffer com.ibm.commerce.order.objects.OfferAccessBean
 */
private void setOffer(com.ibm.commerce.order.objects.OfferAccessBean newOffer) {
	iOffer = newOffer;
}
/**
 * setOfferID method comment.
 */
private void setOfferId(Long anOfferId) {
	iOfferId = anOfferId;
}
/**
 * (Optional) Sets the offer IDs to use for the price.  If no offer IDs are supplied, then the "best" of all
 * qualifying offers will be used.  If offer IDs are specified, the "best" of the specified offers will be returned.
 * @param anOfferIdList the offer IDs.
 */
public void setOfferIds(java.lang.Long[] anOfferIdList) {
	iOfferIds = anOfferIdList;
}
/**
 * (Optional) Set the order item.
 * @param anOrderItemAccessBean an order item access bean. 
 */
public void setOrderItemAccessBean(OrderItemAccessBean anOrderItemAccessBean) {
	iOrderItemAccessBean = anOrderItemAccessBean;
}
/**
 * Insert the method's description here.
 * @param newPrice java.lang.String
 */
private void setPrice(MonetaryAmount aPrice) throws ECSystemException {
	// round customize the price
	CurrencyManager.getInstance().roundCustomized(aPrice, getStoreAB(), NumberUsageConstants.COMMERCE_UNIT_PRICE);
	iPrice = aPrice;
}
/**
 * (Optional) Set the quantity amount of the catalog entry that is to be priced.
 * The quantity amount helps determine which offers are valid.  If the quantity
 * amount is not specified then the nominal quantity (rounded up to the nearest
 * multiple as specified in the quantityMultiple attribute) of the Catalog entry
 * is assumed to be the requested quantity.
 * @param aQuantityAmount a quantity amount.
 */
public void setQuantity(com.ibm.commerce.price.utils.QuantityAmount newQuantityAmount) {
	iQuantityAmount = newQuantityAmount;
}
/**
 * Insert the method's description here.
 * @param newStoreAB com.ibm.commerce.common.objects.StoreAccessBean
 */
private void setStoreAB(com.ibm.commerce.common.objects.StoreAccessBean newStoreAB) {
	iStoreAB = newStoreAB;
}
/**
 * (Optional) Set the store Id to use for calculating the price.  If the store Id 
 * is not specified then the store Id of the command context will be used.
 * @param aStoreId the store Id.
 */
public void setStoreId(java.lang.Integer newStoreId) {
	iStoreId = newStoreId;
}
/**
 * Check mandatory parameters.
 */
public void validateParameters() throws ECException {
	final String methodName = "checkParameters";
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);

	// check the store object - ensure that the storeId is a valid store
	if (getStoreAB() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, iClassName, methodName, new Object[] {ECConstants.EC_STORE_ID});
	}

	// check the catalog entry ID
	if (getCatEntryId() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, iClassName, methodName, new Object[] {OrderConstants.EC_CATALOG_ENTRY_ID});
	}

	// check the currency
	if (getCurrency() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, iClassName, methodName, new Object[] {ECConstants.EC_CURRENCY_ID});
	}
	
	//
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, iClassName, methodName);
}

/**
 * Get the quantity unit only.
 * Creation date: (12/11/2001 10:02:51 AM)
 * @return The quantity unit.
 */
private String getQuantityUnit() {
	String result = null;
	
	// if the quantity was supplied
	if (iQuantityAmount != null) {
		result = iQuantityAmount.getQuantityUnit();
	}

	//
	return result;
}

/**
 * Get the quantity value only.
 * Creation date: (12/11/2001 10:02:51 AM)
 * @return double
 */
private double getQuantityValue() {
	double result = 1.0;

	// if the quantity was supplied
	if (iQuantityAmount != null) {
		result = iQuantityAmount.getValue();
	}

	//
	return result;
}
}
