//********************************************************************
//*-------------------------------------------------------------------
//* Sample Source
//*
//* 5724-A18
//*
//* (c) Copyright IBM Corp. 2000, 2002
//*
//* The source code for this program is not published or otherwise
//* divested of its trade secrets, irrespective of what has been
//* deposited with the US Copyright Office.
//*-------------------------------------------------------------------
//*
////////////////////////////////////////////////////////////////////////////////

import com.ibm.commerce.price.commands.*;


import java.util.*;
import java.rmi.*;
import java.sql.*;
import java.math.*;
import javax.ejb.*;
import javax.naming.*;
import com.ibm.commerce.order.objects.*;
import com.ibm.commerce.order.objimpl.*;
import com.ibm.commerce.exception.*;
import com.ibm.commerce.ras.*;
import com.ibm.commerce.price.utils.*;
import com.ibm.commerce.command.*;
import com.ibm.commerce.common.objects.*;
import com.ibm.commerce.server.*;
import com.ibm.commerce.order.utils.*;
import com.ibm.commerce.user.objects.*;
import com.ibm.commerce.common.objimpl.*;
import com.ibm.commerce.catalog.objects.*;
// PriceList:
import com.ibm.commerce.contract.objects.*;
import com.ibm.commerce.contract.objimpl.*;
import com.ibm.commerce.contract.helper.*;
import com.ibm.commerce.contract.util.*;
import com.ibm.commerce.fulfillment.objects.*;
import com.ibm.commerce.datatype.*;
import com.ibm.commerce.registry.*;

/**
 * This <code>GetContractUnitPriceCmd</code> Task Command Implementation can be called 
 * by several commands such as <code>ProductDisplayCmd</code>, <code>OrderItemAddCmd</code>,
 * <code>OrderItemUpdateCmd</code>, <code>OrderItemDisplayCmd</code> and
 * <code>OrderPrepareCmd</code>,Controller Command to calculate the best contract
 * price for a catalog entry given a list of trading agreements (and offers) and 
 * also for each of these trading agreements, obtain its best contract price.
 * 
 * <p>
 * If the command is successful retrieving the price, it will return the best price,
 * the offer id and access bean, the T&C id and the trading agreement id that
 * determined the price.
 * <p>
 * If unsuccessful, the command throws the ECApplicationException specifying 
 * an Error Message (ERR_RETRIEVE_PRICE), an Error Code (ERR_CODE_NO_PRICE) and 
 * the Error View (RetrievePriceErrorView).
 *
 * <p><strong>Behaviour</strong>
 * <p>Obtain all the qualified offers for the input catalog entry, each offer 
 * to be retrieved to find the price of a catalog entry must satisfy the following
 * criteria (Invoke <code>CalculateContractPricesCmd</code>):
 * <ul>
 * <li>the catalog entry must not be excluded by the input and eligible trading
 * agreements.
 * <li>offer id matches one of the supplied offer ids or no offer ids were supplied.
 * <li>the offer must be associated with a price list that is referenced by one of
 * the eligible trading agreements.
 * <li>the trading agreement id provided the offer matches one of the supplied trading
 * ids or if no trading ids were supplied, matches one of those trading agreements 
 * which are active and also eligible for the specified store and the user.
 * <li>the offer must be published.
 * <li>the current timestamp of the offer must be in the active range specified for 
 * the offer (between the STARTDATE and ENDDATE).
 * <li>either no quantityAmount was specified or the quantity range specified on the
 * offer includes the quantity amount specified. 
 * If a conversion is required and cannot be performed (because there is no conversion 
 * rule) then the offer is not considered.
 * <li>the offer has a valid offer price in the correct currency or a conversion exists
 * from the offer price currency to the requested currency.  If there is more than one 
 * currency that can be converted to the requested currency then the lowest price will
 * be used from all the conversions.
 * If no valid offer can be found, the parent product offers will be searched providing
 * the catalog entry and the parent product is not excluded from the trading agreements.
 * </ul>
 * <p>Get the best price among all the qualified offers for the input catalog entry
 * (Invoke <code>SelectContractPricesCmd</code>):
 * <ul>
 * <li>the price list provided the offer has the highest precedence among all the
 * qualified price lists referenced by all the eligible trading agreements.
 * <li>the offer has the highest possible precedence within its price list.
 * A lower precedence offer may be chosen if there is no qualifying offer with a 
 * higher precedence level in the same price list.
 * <li>the result offer price gives the lowest price, after applying the
 * adjustment specified in the Ts&Cs from the trading agreements.
 * </ul>
 *
 * <p><strong>Exception Conditions</strong>
 * <ul>
 * <li>If no eligible trading agreements can be obtained, the command throws the
 * ECApplicationException specifying an Error Message (ERR_NO_ELIGIBLE_TRADING).
 * <li>If the specified trading agreement is not valid, the command throws the
 * ECApplicationException specifying an Error Message (ERR_NO_VALID_TRADING).
 * <li>If price list cannot be resolved from the price business policies, the command 
 * throws the ECApplicationException specifying an Error Message 
 * (ERR_FOUND_PRICE_LIST_FROM_POLICY).
 * <li>If price list cannot be retrieved, the command throws the ECApplicationException 
 * specifying an Error Message (ERR_RETRIEVE_PRICE_LIST).
 * <li>If no price is found, the command throws the ECApplicationException specifying 
 * an Error Message (ERR_RETRIEVE_PRICE).
 * <p>If ErrorMode is false, these errors will be ignored and a null value price
 * will be passed back to the caller.
 * </ul>
 */
 
public class GetContractUnitPriceCmdImpl extends com.ibm.commerce.command.TaskCommandImpl implements GetContractUnitPriceCmd, GetContractSpecialPriceCmd, GetProductContractUnitPriceCmd {

	private static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private static final String CLASS_NAME				= "com.ibm.commerce.price.commands.GetContractUnitPriceCmdImpl";
	private static final String CHECK_PARAMETERS		= "validateParameters";
	private static final String PERFORM_EXECUTE			= "performExecute";
	private static final String RESOLVE_INPUT_PARAMETERS    = "resolveInputParameters";
	private static final String RESOLVE_TRADINGAGREEMENTS   = "resolveTradingAgreements";
	private static final String CALCULATE_PRICES		    = "calculatePrices";
	private static final String RETRIEVE_PRICES		        = "retrievePrices";	
	private static final String RESOLVE_INPUT_FROM_CATENTRY = "resolveInputFromCatEntryPriceInfo";	
	private static final String RESOLVE_INPUT_FROM_ORDITEMS = "resolveInputFromOrderItems";
	private static final String RETRIEVE_PRICELISTINFO      = "retrievePriceListInfo";
	private static final String SELECT_PRICES			    = "selectPrices";
	private static final String RESOLVE_PRICELISTS			= "resolvePriceLists";
	private static final String	CREATE_CMD_PRICELIST		= "createCmdPriceList";
	private static final String CREATE_ITEM_EXCLUSION_LIST  = "createItemExclusionList";
	private static final String CREATE_ITEM_INCLUSION_LIST  = "createItemInclusionList";
	private static final String CREATE_CMD_MASTERCATALOG_PRICELIST = "createCmdMasterCatalogPriceList";

	private Long inCatEntryId 							= null;
	private CatEntryPrices[] iCatEntryPrices			= null;
	private OrderItemAccessBean iOrderItemAB	 		= null;
	private String istrCurrency 						= null;
	
	private QuantityAmount iQuantityAmount 				= null;
	private CatalogEntryShippingAccessBean iCatalogEntryShippingAB = null;

	private Long inMemberId								= null;
	private boolean iErrorMode							= true;
	
	private ItemPriceInfo[] iItemPriceInfo				= null;
	private Long[] iCatEntryIdsForSearch				= null;
	
	private Long[] iTradingIdsIn 						= null;
	private Long[] iGlobalTradingIds					= null;
	private Long[] iCommandContextTradingIds			= null; 
	private TradingAgreementAccessBean[] iTradingABs  	= null;
	
	private Long[] iOfferIdsIn 							= null;
	private Long[] iGlobalOfferIds						= null;
	 	
	private Long[] iMasterCatalogPriceLists				= null;
	private Hashtable iCmdPriceListHash					= null;
	private Hashtable iCalculatePriceCmdHash			= null;
	
	private boolean iUseGlobalTrading					= true;
	private boolean iUseGlobalOffer						= true;
		
	private MonetaryAmount iPrice 						= null;
	private MonetaryAmount iBasePrice 					= null;
	private Long inTradingIdOut 						= null;
	private Long inTcIdOut 								= null;
	private Long inOfferIdOut							= null;
	private OfferAccessBean iOfferOut				 	= null;
	
	private Timestamp iExpiryDateForCache 				= null;
		
	private Integer inStoreId 							= null;
	private StoreAccessBean iStoreAB 					= null;
		
/**
 * GetBaseUnitPriceCmdImpl constructor.
 */
public GetContractUnitPriceCmdImpl() {
	super();
}
/**
 * Retrieves all the qualified offers for the input catalog entry in that each offer
 * must meet a list of criteria.
 * Selects the best adjustment provided by the Ts&Cs and applies the adjustment on
 * the offer prices.
 */ 
protected void calculatePrices(Integer storeId) throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CALCULATE_PRICES);

	try {
		String requestedCurrency = getCurrency();
		
		if(iUseGlobalTrading) {

			CalculateContractPricesCmd calculatePricesCmd = (CalculateContractPricesCmd) CommandFactory.createCommand(CalculateContractPricesCmd.NAME, storeId);
			
			calculatePricesCmd.setTradingABs(iTradingABs);
			for(int i=0; i<iTradingABs.length; i++) {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CALCULATE_PRICES, 
					"trading agreement input to retrieve offers: " + iTradingABs[i].getTradingId().toString());
			}
			
			calculatePricesCmd.setUseGlobalPriceList(iUseGlobalTrading);

			calculatePricesCmd.setMasterCatalogPriceListIds(iMasterCatalogPriceLists);
			
			if(iMasterCatalogPriceLists != null) {
				for(int j=0; j<iMasterCatalogPriceLists.length; j++) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CALCULATE_PRICES, 
						"master catalog price list input to retrieve offers: " + iMasterCatalogPriceLists[j].toString());
				}
			}	
						
			calculatePricesCmd.setItemPriceInfo(iItemPriceInfo);
  			calculatePricesCmd.setCurrency(requestedCurrency);
  			calculatePricesCmd.setCommandContext(commandContext);
  			calculatePricesCmd.setErrorMode(iErrorMode);
  			calculatePricesCmd.setStoreId(getStoreAB().getStoreEntityIdInEJBType());
			calculatePricesCmd.execute();
			
		}

	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, RETRIEVE_PRICELISTINFO, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, RETRIEVE_PRICELISTINFO, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, RETRIEVE_PRICELISTINFO, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, RETRIEVE_PRICELISTINFO, ex);
	}
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CALCULATE_PRICES);
}
/**
 * Obtains the business commands associated with the price policies for the
 * master catalog (PriceTCMasterCatalogWithOptionalAdjustment) and 
 * constructs a list of corresponding price list objects that will be 
 * input to the each of the commands.
 * <p>
 * @param aPolicyPriceProperties The list of price policy ids and their
 * properties associated with the master catalog.
 * @param standardPriceListABs The list of standard price list access beans
 * associated with the master catalog.
 * <p>
 * @return The list of standard price lists associated with the master catalog.
 */
private Long[] createCmdMasterCatalogPriceList(Hashtable aPolicyPriceLists) throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_MASTERCATALOG_PRICELIST);

	Long[] priceListIds = null;
	
	Vector masterPriceListVec = new Vector();
		
	Enumeration policyEnum = aPolicyPriceLists.keys();
	while(policyEnum.hasMoreElements()) {
		String policyId = (String)policyEnum.nextElement();
		Long priceListId = (Long)aPolicyPriceLists.get(policyId);
			
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_MASTERCATALOG_PRICELIST, 
			"policy id and pricelist id: " + policyId.toString() +  "," + priceListId.toString());
			
		if(!masterPriceListVec.contains(priceListId))
			masterPriceListVec.addElement(priceListId);

		RetrievePricesCmd retrievePricesCmd = (RetrievePricesCmd) CommandFactory.createBusinessPolicyCommand(new Long(policyId),RetrievePricesCmd.NAME);
		if(retrievePricesCmd != null) {
			//Class retrievePricesCmdImpl = retrievePricesCmd.getClass();
			String retrievePricesCmdClass = retrievePricesCmd.getClass().getName();
									
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_MASTERCATALOG_PRICELIST, 
				"pricelist id and command: " + priceListId.toString() + retrievePricesCmdClass.toString());
				
				
			Vector prcListVec = (Vector)iCmdPriceListHash.get(retrievePricesCmdClass);
			if(prcListVec == null) {
				prcListVec = new Vector(2);
				iCmdPriceListHash.put(retrievePricesCmdClass, prcListVec);
			}

			if(!prcListVec.contains(priceListId))
				prcListVec.addElement(priceListId);
				
			iCalculatePriceCmdHash.put(retrievePricesCmdClass, retrievePricesCmd);
		}	
				
	}

	priceListIds = (Long[])CalculationCmdHelper.toArray(masterPriceListVec, Long.class);
			
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_MASTERCATALOG_PRICELIST);

	return priceListIds;
}
/**
 * Obtains the business commands associated with the price policy for a 
 * non master catalog Ts&Cs and stores the price list ids that will be input
 * to each of the the business command.
 * <p>
 * @param aPolicyPriceProperties The list of price policy ids and their
 * properties associated with the price T&C.
 * @param aPriceListId The id of the price list assocated with the price T&C. 
 */
private void createCmdPriceList(Hashtable aPolicyPriceLists, Long aPriceListId) throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_PRICELIST);
	
	//try {

		Enumeration policyEnum = aPolicyPriceLists.keys();
		while(policyEnum.hasMoreElements()) {
			Long policyId = (Long)policyEnum.nextElement();
			Long prcId = (Long)aPolicyPriceLists.get(policyId);
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_PRICELIST, 
				"policy id and priceList id: " + policyId.toString() +  "," + prcId.toString());
			
			if(aPriceListId.equals(prcId)) {
				RetrievePricesCmd retrievePricesCmd = null;
				
				retrievePricesCmd = (RetrievePricesCmd) CommandFactory.createBusinessPolicyCommand(policyId,RetrievePricesCmd.NAME);
				if(retrievePricesCmd != null) {
					//Class retrievePricesCmdImpl = retrievePricesCmd.getClass();
					String retrievePricesCmdClass = retrievePricesCmd.getClass().getName();
								
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_PRICELIST, 
						"command and priceList id: " + retrievePricesCmdClass.toString() + "," + aPriceListId.toString());
				
					// PriceList?: assuming that all price policies for the same price list use the same
					//             command class
					Vector prcListVec = (Vector)iCmdPriceListHash.get(retrievePricesCmdClass);
					if(prcListVec == null) {
						prcListVec = new Vector(2);
						iCmdPriceListHash.put(retrievePricesCmdClass, prcListVec);
					}

					if(!prcListVec.contains(aPriceListId))
						prcListVec.addElement(aPriceListId);
				
					iCalculatePriceCmdHash.put(retrievePricesCmdClass, retrievePricesCmd);
				}	

				break;
			}
				
		}
		
	//} catch (javax.ejb.CreateException ex) {
		//throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	//} catch (javax.ejb.FinderException ex) {
	//	throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	//} catch (javax.naming.NamingException ex) {
	//	throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, PERFORM_EXECUTE, ex);
	//} catch (java.rmi.RemoteException ex) {
	//	throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	//}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CREATE_CMD_PRICELIST);
}
/**
 * Generates a search query to exclude a list of product sets which has been
 * specified in the trading agreements.
 * <p>
 * @param aTradingABs a list of trading agreement access beans.
 * <p>
 * @return The product set exclusion search query string.
 */
private Integer[] createItemExclusionList(TradingAgreementAccessBean[] aTradingABs) throws ECException {

	Integer[] exclusionPsIds = null;
	
	try {
		ProductSetSelection psSelection = new ProductSetSelection(aTradingABs, commandContext.getUserId());
		Vector psExclusion = psSelection.getExclusion();

		if(psExclusion != null) {
			exclusionPsIds = (Integer[])CalculationCmdHelper.toArray(psExclusion, Integer.class);			
		}		
		
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, CREATE_ITEM_EXCLUSION_LIST, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, CREATE_ITEM_EXCLUSION_LIST, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, CREATE_ITEM_EXCLUSION_LIST, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, CREATE_ITEM_EXCLUSION_LIST, ex);
	}

	return exclusionPsIds;
	
}
/**
 * Generates a search query to include a list of product sets which has been
 * specified in the trading agreements.
 * <p>
 * @param aTradingABs a list of trading agreement access beans.
 * <p>
 * @return The product set inclusion search query string.
 */
private Integer[] createItemInclusionList(TradingAgreementAccessBean[] aTradingABs) throws ECException {

	Integer[] inclusionPsIds = null;
	
	try {
		ProductSetSelection psSelection = new ProductSetSelection(aTradingABs, commandContext.getUserId());
		Vector psInclusion = psSelection.getInclusion();

		if(psInclusion != null) {
			inclusionPsIds = (Integer[])CalculationCmdHelper.toArray(psInclusion, Integer.class);			
		}	
		
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, CREATE_ITEM_INCLUSION_LIST, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, CREATE_ITEM_INCLUSION_LIST, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, CREATE_ITEM_INCLUSION_LIST, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, CREATE_ITEM_INCLUSION_LIST, ex);
	}

	return inclusionPsIds;
	
}
/** 
 * Creates a list of trading agreement ids and offer ids for global use 
 */
private void createTradingOfferList() throws ECException {

	final String strMethod = "createTradingOfferList";
	
	if(iUseGlobalTrading) {
		// PriceList: use the global trading values
		if(iTradingIdsIn != null && iTradingIdsIn.length > 0) {
			iGlobalTradingIds = new Long[iTradingIdsIn.length];
			iGlobalTradingIds = iTradingIdsIn;
		}
		else if(iCommandContextTradingIds != null) {
			// PriceList: use those from Command Context
			iGlobalTradingIds = new Long[iCommandContextTradingIds.length];
			iGlobalTradingIds = iCommandContextTradingIds;
		}	
	}
	else {
		Vector tradingIdsVec = null;
		
		if(iItemPriceInfo != null) {
			tradingIdsVec = new Vector(5);
			for(int i=0; i<iItemPriceInfo.length; i++) {
				Long[] tradeIds = iItemPriceInfo[i].getTradingIds();
				for(int j=0; j<tradeIds.length; j++) {
					if(!tradingIdsVec.contains(tradeIds[j])) {
						tradingIdsVec.addElement(tradeIds[j]);
					}	
				}
			}
		}

		if(tradingIdsVec != null) 
			iGlobalTradingIds = (Long[])CalculationCmdHelper.toArray(tradingIdsVec, Long.class);
		
	}	
		
	if(iUseGlobalOffer) {
		// PriceList: use the global offer values
		if(iOfferIdsIn != null && iOfferIdsIn.length > 0) {
			iGlobalOfferIds = new Long[iOfferIdsIn.length];
			iGlobalOfferIds = iOfferIdsIn;
		}	
	}	
	
}
/**
 * Returns the ids of the trading agreements that are used to determine the prices.
 * <p>
 * @return A list of trading agreement ids for determining the prices.
 */
public Long[] getApplicableTradingIds() {
	
	if(iCatEntryPrices != null && iCatEntryPrices.length > 0)
		return iCatEntryPrices[0].getAllTradingIds();
	
	return null;
}
/**
 * Returns the unit prices of a catentry that were determined for each of the 
 * applied trading agreements.
 * <p>
 * @return A list of unit prices of a catentry for each of the applied trading
 * agreements.
 */
public MonetaryAmount[] getApplicableTradingUnitPrices() {
		
	if(iCatEntryPrices != null && iCatEntryPrices.length > 0)
		return iCatEntryPrices[0].getTradingUnitPrices();
	
	return null;
}
/**
 * Returns the base price of the input catalog entry.
 */
public MonetaryAmount getBasePrice() {
	return iBasePrice;
}
/**
 * Returns the id of the input catalog entry.
 */
private Long getCatEntryId() {
	return inCatEntryId;
}
/**
 * Returns the price related information of the input catalog entries.
 */
public CatEntryPrices[] getCatEntryPrices() { return iCatEntryPrices; }
/**
 * Returns the currency to be used for the price.
 */
private java.lang.String getCurrency() throws ECSystemException {
	if (istrCurrency == null) {
		CommandContext context = getCommandContext();

		//
		istrCurrency = Helper.getCurrency( context, getStoreAB() );
	}
	return istrCurrency;
}
/**
 * Returns the internal item info structure.
 */
private ItemPriceInfo[] getItemPriceInfo() { return iItemPriceInfo; }
/**
 * Returns the id of the input member that was used to verify and determine 
 * the trading agreements.
 */
private Long getMemberId() {
	return inMemberId;
}
/**
 * Returns the offer access bean that was used to determine the price.
 */
public OfferAccessBean getOffer() {
	return iOfferOut;
}
/**
 * Returns the id of the output offer id that was used to determine the price.
 */
public Long getOfferId() {
	return inOfferIdOut;
}
/**
 * Returns the ids of the input offers to be used for price.
 */
private Long[] getOfferIds() {
	return iOfferIdsIn;
}
/**
 * Returns the orderitem access bean.
 */
private OrderItemAccessBean getOrderItemAccessBean() {
	return iOrderItemAB;
}
/**
 * Returns the actual price of the catalog entry. 
 */
public MonetaryAmount getPrice() {
	return iPrice;
}
private com.ibm.commerce.price.utils.QuantityAmount getQuantity() { 
	return iQuantityAmount;
}	
/**
 * Returns the quantity amount of the input catalog entry for price.
 * If the quantity amount is not specified then the nominal quantity
 * (rounded up to the nearest multiple as specified in the quantityMultiple attribute)
 * of the catalog entry is assumed to be the requested quantity.
 */
private com.ibm.commerce.price.utils.QuantityAmount getQuantityAmount() throws ECSystemException {
	// if the quantity was not supplied, use the rounded quantity from the catalog entry shipping record
	if (iQuantityAmount == null) {
		iQuantityAmount = getRoundedQuantityAmount(1.0, inCatEntryId);
	}

	//
	return iQuantityAmount;
}
/**
 * Returns the quantity amount that has been rounded up for the input catalog entry.
 * The amount is rounded up to the nearest multiple as specified in the
 * quantityMultiple attribute of the catalog entry
 * <p>
 * @param aQuantity the quantity to be converted.
 * @param aCatEntryId the input catalog entry.
 * <p>
 * @return The rounded up quantity amount for the catalog entry.
 */
private QuantityAmount getRoundedQuantityAmount(double aQuantity, Long aCatEntryId) throws ECSystemException {
	QuantityAmount quantityAmount;
	
	if( iCatalogEntryShippingAB == null ) {
		iCatalogEntryShippingAB = Helper.getCatalogEntryShippingAB(aCatEntryId);
	}

	quantityAmount = Helper.createRoundedQuantityAmount(iCatalogEntryShippingAB, aQuantity);

	return quantityAmount;
}
/**
 * Returns the store access bean.
 *
 * If the store id is not specified then the store id of the command context
 * will be used and the corresponding access bean will be retrieved from the
 * registry.
 */
private com.ibm.commerce.common.objects.StoreAccessBean getStoreAB() {
	if( iStoreAB == null ) {
		Integer storeId = myGetStoreId();
		if( storeId != null ) {
			setStoreAB(WcsApp.storeRegistry.find(storeId));
		}
	}
	return iStoreAB;
}
/**
 * Returns the id of the output T&C id which was associated with the offer
 * that supplied the price.
 */
public Long getTcId() {
	return inTcIdOut;
}
/**
 * Returns the id of the output trading agreement id which was associated with the offer 
 * that supplied the price.
 */
public Long getTradingId() {
	return inTradingIdOut;
}
/**
 * Returns the ids of the input trading agreements to be used for price.
 */
private Long[] getTradingIds() throws ECSystemException {
	return iTradingIdsIn;
}
/**
 * Verifies the input trading agreement against a list of eligible trading agreements.
 * <p>
 * @param anTradingId the id of the trading agreement to be verified.
 * @param aEligibleTradingIds the ids of the list of eligible trading agreements.
 * <p>
 * @return true if the trading agreements is valid; false otherwise.
 */
private boolean isTradingAgreementValid(Long anTradingId, String[] aEligibleTradingIds) throws ECException {

	final String strMethod = "isTradingAgreementValid";
	
	if(aEligibleTradingIds != null && aEligibleTradingIds.length > 0) {
		for(int i=0; i<aEligibleTradingIds.length; i++) {
			 if(anTradingId.toString().equalsIgnoreCase(aEligibleTradingIds[i])) {
				 ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod,
					 "input trading agreement is valid: " + anTradingId.toString());
			 	return true;
			 }	
		}
	}
	else {
		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod,
					 "input trading agreement cannot be verified: " + anTradingId.toString());
		return false;
	}	

	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod,
					 "input trading agreement is invalid: " + anTradingId.toString());
	return false;		 	
					
}
/**
 * Resolves the store id. If the store id is not specified then the store id of
 * the command context will be used.
 * <p>
 * @return The store id to be used.
 */
private java.lang.Integer myGetStoreId() {
	if( inStoreId == null ) inStoreId = getStoreId();
	return inStoreId;
}
/**
 * Executes main business logic of the command.
 */
public void performExecute() throws ECException {
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, PERFORM_EXECUTE);

	// check that all parameters are supplied
	//checkParameters();

	try {
		Integer storeId = getStoreAB().getStoreEntityIdInEJBType();

		// verify and resolve all input parameters 
		if(resolveInputParameters()) {
			
			if(iGlobalTradingIds != null) {
				// obtain all the price lists and their corresponding policy information
				retrievePriceListInfo();

				// retrieve all qualified prices from the price lists   
				retrievePrices();
			}	
			//else {
				// if not global, process each item individually
			//}		
				
			// calculate the prices with adjustment (based on offers)
			calculatePrices(storeId);

			// select the best prices (based on offers)
			selectPrices(storeId);
		}	
   
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, PERFORM_EXECUTE, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, PERFORM_EXECUTE, ex);
	}
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, PERFORM_EXECUTE);
}
/**
 * reset the command.
 */
public void reset() {
	inCatEntryId 			= null;
	iCatEntryPrices			= null; 
	iOrderItemAB			= null;
	istrCurrency 			= null;

	iQuantityAmount 		= null;
	iCatalogEntryShippingAB = null;

	iTradingIdsIn 			= null;
	iOfferIdsIn 			= null;

	inMemberId				= null;
	iErrorMode				= true;
		
	iPrice 					= null;
	iBasePrice 				= null;
	inTradingIdOut   		= null;
	inTcIdOut 				= null;
	inOfferIdOut			= null;
	iOfferOut				= null;
	
	iExpiryDateForCache 	= null;
			
	
	
	
	inStoreId 				= null;
	iStoreAB 				= null;
}
/**
 * Constructs the internal item info structure with the catentry & other parameters 
 */
private void resolveInputFromCatEntry() throws ECException {

	final String strMethod = "resolveInputFromCatEntry";

	iItemPriceInfo = new ItemPriceInfo[1];
	iItemPriceInfo[0] = new ItemPriceInfo(inCatEntryId,getQuantity());

	iCatEntryIdsForSearch = new Long[1];
	iCatEntryIdsForSearch[0] = inCatEntryId;
										
	if(iTradingIdsIn != null && iTradingIdsIn.length > 0) {
		iItemPriceInfo[0].setTradingIds(iTradingIdsIn);
		iItemPriceInfo[0].initOutput(iTradingIdsIn.length);
	}	
	else if(iCommandContextTradingIds != null) {
		iItemPriceInfo[0].setTradingIds(iCommandContextTradingIds);
		iItemPriceInfo[0].initOutput(iCommandContextTradingIds.length);
	}		
	
	// PriceImprove: set the trading values 	
	//if(iTradingABs != null) {	
	//	for(int i=0; i<iTradingABs.length; i++) {
			//iCommonTradingIds[i] = iTradingABs[i].getTradingIdInEJBType();
	//	}	
	//}
		
	// PriceList: set the offer values
	if(iOfferIdsIn != null && iOfferIdsIn.length > 0) {
		iItemPriceInfo[0].setOfferIds(iOfferIdsIn);
	}
					
}
/**
 * Constructs the internal item info structure with CatEntryPrices and other input parameters (Future Support)
 */
private void resolveInputFromCatEntryPrices() throws ECException {

	final String strMethod = "resolveInputFromCatEntryPrices";
	
	iItemPriceInfo = new ItemPriceInfo[iCatEntryPrices.length]; 	
	iCatEntryIdsForSearch = new Long[iCatEntryPrices.length];
				
	for(int j=0; j<iCatEntryPrices.length; j++) {
		iItemPriceInfo[j] = new ItemPriceInfo(iCatEntryPrices[j].getCatalogEntryId(),
			                           iCatEntryPrices[j].getQuantity());
		
		iCatEntryIdsForSearch[j] = iCatEntryPrices[j].getCatalogEntryId();
			
		Long[] tIds = iCatEntryPrices[j].getTradingIds();

		if(tIds != null) {
			iUseGlobalTrading = false;

			String[] eligibleTradingIds = commandContext.getEligibleTradingAgreementIds();
			if(eligibleTradingIds != null && eligibleTradingIds.length > 0) {
				for(int i=0; i<eligibleTradingIds.length; i++) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, 
						"eligible trading agreement from Command Context: " + eligibleTradingIds[i].toString());
				}
			}
			else {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, 
						"no eligible trading agreement can be found from Command Context");
			}
			
			for(int i=0; i<tIds.length; i++) {	
				if(!isTradingAgreementValid(tIds[i], eligibleTradingIds)) {
					throw new ECApplicationException(ECMessage._ERR_NO_ELIGIBLE_TRADING, this.getClass().getName(), RESOLVE_INPUT_FROM_CATENTRY);
				}
			}	
			
			iItemPriceInfo[j].setTradingIds(tIds);
			iItemPriceInfo[j].initOutput(tIds.length);
		}
		else {
			// PriceList: use the global trading values
			if(iTradingIdsIn != null && iTradingIdsIn.length > 0) {
				iItemPriceInfo[j].setTradingIds(iTradingIdsIn);
				iItemPriceInfo[j].initOutput(iTradingIdsIn.length);
			}
			else if(iCommandContextTradingIds != null) {
				// PriceList: use those from Command Context
				iItemPriceInfo[j].setTradingIds(iCommandContextTradingIds);
				iItemPriceInfo[j].initOutput(iCommandContextTradingIds.length);
			}	
		}		
					
		Long[] oIds = iItemPriceInfo[j].getOfferIds();

		if(oIds != null) {
			iUseGlobalOffer = false;

			iItemPriceInfo[j].setOfferIds(oIds);	
		}
		else {
			// PriceList: use the global offer values
			if(iOfferIdsIn != null) {
				iItemPriceInfo[j].setOfferIds(iOfferIdsIn);
			}	
		}
	}	
	
}
/**
 * Verifies and resolves the input parameters: catentries, trading agreements and offers.
 * If trading agreements are not provided, a list of eligible trading agreements
 * will be obtained.
 * <p>
 * @return true if all input parameters are valid and valid trading agreements 
 * can be resolved.
 * <p> 
 * @exception ECApplicationException If no eligible trading agreements can be
 * obtained (ERR_NO_ELIGIBLE_TRADING). 
 */
private boolean resolveInputParameters() throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_INPUT_PARAMETERS);

	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_INPUT_PARAMETERS, "currency used for prices: " + getCurrency());
	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_INPUT_PARAMETERS, "input error mode: " + iErrorMode);
	
	// callers can pass in one of the following combinations of parameters:
	//  1. CatEntryPrices which include madatory specific TradingIds and OfferIdsIns (future)
	//  2. CatEntryId, TradingIdsIns (optional global), OfferIdsIns (optional global)
							
	// individual catentry being passed in from e.g. ProductDisplay, OrderItemAdd
	if(iCatEntryPrices == null) {
		if(inCatEntryId != null) {
			if(!resolveTradingAgreements()) return false;
				resolveInputFromCatEntry();
		}	
	}
	else if(iCatEntryPrices.length > 0) {
		// reserved for future support
		//resolveInputFromCatEntryPrices();					
	}

	createTradingOfferList();

	if(iUseGlobalTrading && (iGlobalTradingIds == null || iGlobalTradingIds.length <= 0)) {
		// raise an exception
		throw new ECApplicationException(ECMessage._ERR_NO_ELIGIBLE_TRADING, this.getClass().getName(), RESOLVE_INPUT_PARAMETERS);
	}	
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_INPUT_PARAMETERS);
	
	return true;	
}
/**
 * Resolve the price lists referenced by a list of price business policies.
 *
 * @param pricePolicyABs the access beans of price business policies.
 * <p>
 * @return A list of price lists associated with each of the input price policies.
 */ 
protected Hashtable resolvePriceLists(BusinessPolicyAccessBean[] pricePolicyABs) throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_PRICELISTS);

	Hashtable policyPriceLists = null;
	
	try {
		
		if(pricePolicyABs != null && pricePolicyABs.length > 0) {

			//PriceList?: take the first policy only
			Long policyId = pricePolicyABs[0].getPolicyIdInEJBType();
			
			ResolvePriceListsCmd resolvePriceListsCmd = (ResolvePriceListsCmd) CommandFactory.createBusinessPolicyCommand(policyId, ResolvePriceListsCmd.NAME);
		
			if(resolvePriceListsCmd != null) {
				resolvePriceListsCmd.setPricePolicyABs(pricePolicyABs);
		
				resolvePriceListsCmd.setCommandContext(commandContext);
		
				resolvePriceListsCmd.execute();
			
				policyPriceLists = resolvePriceListsCmd.getPolicyPriceLists();
			}
		}
				
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, RESOLVE_PRICELISTS, ex);
	}		

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_PRICELISTS);

	return policyPriceLists;
		
}
/**
 * Verifies and resolves a list of eligible trading agreements for the user.
 * <p>If trading agreements are specified, the input list will be verified 
 * against the eligible list when the member id is provided.  
 * The eligible list of trading agreements can be obtained from
 * the Command Context if the input member id is the one in Command Context.
 * <p>If trading agreements are not specified, the list may be obtained 
 * directly from the Command Context (if no member id is provided or the
 * input member id is the one in Command Context) or retrieved by means of
 * the member id (if the id is provided and is different from that in the
 * Command Context).
 * <p>
 * @return true if valid trading agreements can be resolved.
 * <p> 
 * @exception ECApplicationException If no eligible trading agreements can be 
 * obtained (ERR_NO_ELIGIBLE_TRADING). 
 * @exception ECApplicationException If the specified trading agreement is not
 * valid (ERR_NO_VALID_TRADING). 
 */
protected boolean resolveTradingAgreements() throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS);
	
	try {
		
		CommandContext commandContext = getCommandContext();
		
		// obtain the list of trading agreements by means of one of the following ways:
		//  1. using the trading agreements provided by the caller 
		//  2. using the trading agreements from CommandContext which may be
		//     - the trading agreements set by the user (set in current session by Runtime)
		//     - all trading agreements entitled for the user (set in current session by Runtime)
		//  3  using all the eligible trading agreements for the user  

		// input trading Ids is provided, verify them if necessary
		if(iTradingIdsIn != null && iTradingIdsIn.length > 0) {

			for(int i=0; i<iTradingIdsIn.length; i++) {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, "trading agreement from input to be used: " + iTradingIdsIn[i].toString());
			}

			// if MemberId is provided, perform validation based on the member Id
			String[] eligibleTradingIds = null;
			
			if(getMemberId() != null) {
								
				if(commandContext.getUserId().equals(getMemberId())) 	
					eligibleTradingIds = commandContext.getEligibleTradingAgreementIds();
				else {
					Vector tradingIdVec = new Vector();
					TradingAgreementAccessBean tradingAB = new TradingAgreementAccessBean();
					Enumeration tradingEnum = tradingAB.findEntitledTradingAgreementByTypeForBuyerUnderStore(ECContractConstants.EC_TRADING_TYPE_CONTRACT, getMemberId(), commandContext.getStoreId());
					while(tradingEnum.hasMoreElements()) {
   						tradingAB = (TradingAgreementAccessBean)tradingEnum.nextElement();
   						tradingIdVec.addElement(tradingAB.getTradingId());
					}	
				
					eligibleTradingIds = (String[])CalculationCmdHelper.toArray(tradingIdVec, String.class);
				}	
					
				if(eligibleTradingIds != null && eligibleTradingIds.length > 0) {
					for(int i=0; i<eligibleTradingIds.length; i++) {
						ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, 
							"eligible trading agreement from Command Context: " + eligibleTradingIds[i].toString());
					}
				}
				else {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, 
							"no eligible trading agreement can be found from Command Context.");
				}
			}	

			
			iTradingABs = new TradingAgreementAccessBean[iTradingIdsIn.length];
						
			for(int i=0; i<iTradingIdsIn.length; i++) {

				if(getMemberId() != null) {
					if(!isTradingAgreementValid(iTradingIdsIn[i], eligibleTradingIds)) {
						if(iErrorMode) {
							Object[] msgParms = {iTradingIdsIn[i].toString()}; 
							throw new ECApplicationException(ECMessage._ERR_INVALID_TRADING, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, msgParms);
						}
						else {
							return false;
						}
					}		
				}
				
				//TradingAgreementAccessBean tradingAB = new TradingAgreementAccessBean();
				//tradingAB.setInitKey_tradingId(iTradingIdsIn[i].toString());
				//tradingAB.refreshCopyHelper();
				TradingAgreementAccessBean tradingABcopy = commandContext.getTradingAgreement(iTradingIdsIn[i]);
				iTradingABs[i] = tradingABcopy; 	
			}
							
		}
		else {
			// input trading ids is not provided, retrieve them from Command Context or elsewhere 

			// if MemberId is provided, retrieve them based on the member Id
			if(getMemberId() != null) {
				
				if(commandContext.getUserId().equals(getMemberId())) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, "obtaining trading agreements from command context.");	
					iTradingABs = commandContext.getCurrentTradingAgreements();
				}
				else {	
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, "obtaining trading agreements from command context for user: " + getMemberId().toString());
				
					Vector tradingABsVec = new Vector();
			
					//iTradingIdsVec.addElement(OrderHelper.getDefaultTradingId(iStoreAB.));
					TradingAgreementAccessBean tradingAB = new TradingAgreementAccessBean();
					Enumeration tradingEnum = tradingAB.findEntitledTradingAgreementByTypeForBuyerUnderStore(ECContractConstants.EC_TRADING_TYPE_CONTRACT, commandContext.getUserId(), getStoreAB().getStoreEntityIdInEJBType());
				
					while(tradingEnum.hasMoreElements()) {
   						tradingAB = (TradingAgreementAccessBean)tradingEnum.nextElement();
   						TradingAgreementAccessBean tradingABcopy = commandContext.getTradingAgreement(tradingAB.getTradingIdInEJBType());
   				 		tradingABsVec.addElement(tradingABcopy);
					}	
				
					iTradingABs = (TradingAgreementAccessBean[])CalculationCmdHelper.toArray(tradingABsVec, TradingAgreementAccessBean.class);
				}	
			}
			else {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, "obtaining trading agreements from command context");	
				iTradingABs = commandContext.getCurrentTradingAgreements();		
			}
				
			if(iTradingABs != null) {	
				iCommandContextTradingIds = new Long[iTradingABs.length];
				
				for(int i=0; i<iTradingABs.length; i++) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, "trading agreements from command context to be used: " + iTradingABs[i].getTradingId());
					iCommandContextTradingIds[i] = iTradingABs[i].getTradingIdInEJBType();
				}	
			}
		}		
			
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS, ex);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RESOLVE_TRADINGAGREEMENTS);

	return true;
}
/**
 * Retrieves the price business policies and price lists referenced by
 * the price Terms & Conditions that are defined in each of the trading agreements
 * <p>Four types of price Terms&Conditions can be defined:
 * <ul>
 * <li>PriceTCMasterCatalogWithOptionalAdjustment:
 * <p>price list objects will be derived from the properties of all of the
 * valid price business policies.  Each object will be validated against
 * the trading position containers in TRADEPOSCN and only standard type
 * price lists will be used.
 * <li>PriceTCPriceListWithOptionalAdjustment
 * <p>the price list id will be directly obtained from the T&C (BIGINTFIELD1).
 * <li>PriceTCPriceListWithSelectiveAdjustment
 * <p>the price list id will be directly obtained from the T&C (BIGINTFIELD1).
 * <li>PriceTCCustomPriceList
 * <p>the price list id will be directly obtained from the T&C (BIGINTFIELD1).
 * </ul>
 * <p>Obtain the business commands associated with the policies and construct a
 * list of corresponding price list objects that will be input to the each of
 * the commands.
 * <p>
 * @exception ECApplicationException If price policies or price lists cannot be obtained
 * or price list cannot be resolved from the policies (ERR_FOUND_PRICE_LIST_FROM_POLICY).
 */
private void retrievePriceListInfo() throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
	
	try {
				
		iCmdPriceListHash = new Hashtable(1);
		iCalculatePriceCmdHash = new Hashtable(1);
			
	   	for(int j=0; j<iGlobalTradingIds.length; j++) {

		   	ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), RETRIEVE_PRICELISTINFO, 
					"retrieving price list for trading agreement: " + iGlobalTradingIds[j].toString());
		   	
	    	TermConditionAccessBean[] priceListTCABs = (TermConditionAccessBean[])iTradingABs[j].getTCsByTCType(PriceCalculationConstants.PriceTCType, commandContext.getUserId());
					    	
			if(priceListTCABs != null) {
				for(int i=0; i<priceListTCABs.length; i++) {

					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), RETRIEVE_PRICELISTINFO, 
					"retrieving price list for T&C: " + priceListTCABs[i].getReferenceNumber() + "(" + priceListTCABs[i].getTcSubType() + ")");
					
					Long priceListId = priceListTCABs[i].getPriceListIdInEJBType();

					if(priceListTCABs[i].getTcSubType().equalsIgnoreCase(PriceCalculationConstants.MasterCatalogWithOptionalAdj)) {
				   		// PriceList: multiple pricelists TC (MasterCatalogTC)
						
						//Hashtable policyPriceProperties = PriceCalculationHelper.getInstance().getAllPolicyPriceProperties(getStoreAB());
						//if(policyPriceProperties != null && policyPriceProperties.size() > 0) {
						//	Vector standardPriceListABVec = PriceCalculationHelper.getInstance().getAllStandardPriceListAccessBeans();
						//	if(standardPriceListABVec != null && standardPriceListABVec.size() > 0) {

						BusinessPolicyRegistryEntry[] policyRegEntries = BusinessPolicyRegistry.singleton().getPolicyRegistryEntryByTypeAndStore(PriceCalculationConstants.PricePolicyType, getStoreAB().getStoreEntityIdInEJBType());
						if(policyRegEntries != null && policyRegEntries.length > 0) {
							BusinessPolicyAccessBean[] pricePolicyABs = new BusinessPolicyAccessBean[policyRegEntries.length]; 
							for(int k=0; k<policyRegEntries.length; k++)
								pricePolicyABs[k] = policyRegEntries[k].getBean();
							
							Hashtable policyPriceLists = resolvePriceLists(pricePolicyABs);
							if(policyPriceLists != null && policyPriceLists.size() > 0) {
								iMasterCatalogPriceLists = createCmdMasterCatalogPriceList(policyPriceLists);
							}
							else {
								throw new ECApplicationException(ECMessage._ERR_FOUND_PRICE_LIST_FROM_POLICY, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
							}		
						}		
						else {
								throw new ECApplicationException(ECMessage._ERR_FOUND_PRICE_LIST_FROM_POLICY, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
						}	
					}
			   		else {
						// PriceList: single priceList TC
						if(priceListId == null) {
							throw new ECApplicationException(ECMessage._ERR_FOUND_PRICE_LIST_FROM_POLICY, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
						}
						else {
							if(!priceListTCABs[i].getTcSubType().equalsIgnoreCase(PriceCalculationConstants.CustomPriceListType)) {
				  				BusinessPolicyAccessBean[] policyABs = priceListTCABs[i].getPoliciesByType(PriceCalculationConstants.PricePolicyType);   
		   	  		 			if(policyABs != null && policyABs.length > 0) {
				  					Hashtable policyPriceLists = new Hashtable();
			   		  	 			policyPriceLists.put(policyABs[0].getPolicyIdInEJBType(), priceListId);
				  					createCmdPriceList(policyPriceLists, priceListId);
		   	  			 		}
		   	  		 			else {
			   	  		 			throw new ECApplicationException(ECMessage._ERR_FOUND_PRICE_LIST_FROM_POLICY, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
		   	  		 			}
							}
							else {
								BusinessPolicyAccessBean policyAB = new BusinessPolicyAccessBean();
								
								try {
									policyAB = policyAB.findCustomPriceListPolicyByStoreEntityId(getStoreAB().getStoreEntityIdInEJBType());
								} catch (javax.ejb.FinderException ex) {
									policyAB = policyAB.findCustomPriceListPolicyByStoreEntityId(getStoreAB().getStoreGroupIdInEJBType());
								}
									
								if(policyAB != null) {
									Hashtable policyPriceLists = new Hashtable();

									Long policyId = policyAB.getPolicyIdInEJBType();
									policyPriceLists.put(policyId, priceListId);
				  					createCmdPriceList(policyPriceLists, priceListId);
								}
								else 
									throw new ECApplicationException(ECMessage._ERR_FOUND_PRICE_LIST_FROM_POLICY, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
							
		   	  			 	}
						}
			   		}		
										
				// end of PriceTC loop	
				}				   	 	
			}
		// end of Trading Id loop	
		}
			
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, this.getClass().getName(), RETRIEVE_PRICELISTINFO, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, this.getClass().getName(), RETRIEVE_PRICELISTINFO, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, this.getClass().getName(), RETRIEVE_PRICELISTINFO, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, this.getClass().getName(), RETRIEVE_PRICELISTINFO, ex);
	}

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), RETRIEVE_PRICELISTINFO);
}
/**
 * Retrieves all the qualified offers for the input catalog entry in that each offer
 * must meet a list of criteria.
 */ 
protected void retrievePrices() throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_PRICES);

	try {
		String requestedCurrency = getCurrency();
		
		Long[] catEntries = null;
		QuantityAmount[] qtyAmounts = null;
		if(iItemPriceInfo != null && iItemPriceInfo.length> 0) {
			catEntries = new Long[iItemPriceInfo.length];
			qtyAmounts = new QuantityAmount[iItemPriceInfo.length];

			for(int i=0; i<iItemPriceInfo.length; i++) {
				catEntries[i] = iItemPriceInfo[i].getCatalogEntryId();
				qtyAmounts[i] = iItemPriceInfo[i].getQuantity();
			}	
				
		}

		Integer[] exclusionList = createItemExclusionList(iTradingABs);
		Integer[] inclusionList = createItemInclusionList(iTradingABs);
			
		Enumeration cmdPrcListEnum = iCmdPriceListHash.keys();

		if(!cmdPrcListEnum.hasMoreElements()) {
			throw new ECApplicationException(ECMessage._ERR_RETRIEVE_PRICE_LIST, CLASS_NAME, RETRIEVE_PRICES);
		}	
		
		while(cmdPrcListEnum.hasMoreElements()) {
			String cmdClassName = (String)cmdPrcListEnum.nextElement();
			Vector prcListVec = (Vector)iCmdPriceListHash.get(cmdClassName);
			Long[] priceList = (Long[])CalculationCmdHelper.toArray(prcListVec, Long.class);
			if(priceList == null || priceList.length <= 0) {
				throw new ECApplicationException(ECMessage._ERR_RETRIEVE_PRICE_LIST, CLASS_NAME, RETRIEVE_PRICES);
			}	
			
			RetrievePricesCmd retrievePricesCmd = (RetrievePricesCmd)iCalculatePriceCmdHash.get(cmdClassName); 

			//calculatePricesCmd.setUseGlobalPriceList(iUseGlobalTrading);
				
			retrievePricesCmd.setPriceListIds(priceList);
			
			for(int j=0; j<priceList.length; j++) {
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_PRICES, 
					"price list input to retrieve offers: " + priceList[j].toString());
			}
						
			if(iUseGlobalOffer) {
				retrievePricesCmd.setOfferIds(iGlobalOfferIds);

				if(iGlobalOfferIds != null) {
					for(int k=0; k<iGlobalOfferIds.length; k++) {
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_PRICES, 
						"offer input to retrieve offers: " + iGlobalOfferIds[k].toString());
					}
				}	
			}

			retrievePricesCmd.setCatEntryIds(catEntries);
			retrievePricesCmd.setQuantities(qtyAmounts);
			retrievePricesCmd.setCurrency(requestedCurrency);
			retrievePricesCmd.setPSExclusionList(exclusionList);
			retrievePricesCmd.setPSInclusionList(inclusionList);
			retrievePricesCmd.setCommandContext(commandContext);
			retrievePricesCmd.setStoreId(getStoreAB().getStoreEntityIdInEJBType());
			retrievePricesCmd.execute();

			if(retrievePricesCmd.getItemOffers() != null) {
				Hashtable[] itemOffers = retrievePricesCmd.getItemOffers();
				for(int i=0; i<itemOffers.length; i++) {
					if(itemOffers[i] != null && itemOffers[i].size() > 0) {
						Hashtable offerInfoHash = (Hashtable)iItemPriceInfo[i].getOffersWithAdjPrice();
		 				if(offerInfoHash == null) {
							offerInfoHash = new Hashtable(2);
							iItemPriceInfo[i].setOffersWithAdjPrice(offerInfoHash);    
		 				}

		 				Enumeration itemOffersEnum = itemOffers[i].elements();
						while(itemOffersEnum.hasMoreElements()) {
							OfferAccessBean offerAB = (OfferAccessBean)itemOffersEnum.nextElement();
							QualifyingOfferInfo offerInfo = new QualifyingOfferInfo();
							offerInfo.setOffer(offerAB);
							offerInfoHash.put(offerAB.getOfferIdInEJBType(), offerInfo);
						}	
												
					}	
				}		
			}
			
		}

	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, RETRIEVE_PRICES, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, RETRIEVE_PRICES, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, RETRIEVE_PRICES, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, RETRIEVE_PRICES, ex);
	}
	
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, RETRIEVE_PRICES);
}
/**
 * Selects the best offer for the input catalog entry among a list of
 * qualified offers.  
 */ 
protected void selectPrices(Integer storeId) throws ECException {

	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, SELECT_PRICES);

	try {
		String requestedCurrency = getCurrency();

		//StoreGetPriceInputData inputData = new StoreGetPriceInputData( getOfferIds(), getContractIds(), getMemberGroupIds() );
		
		SelectContractPricesCmd selectPricesCmd = (SelectContractPricesCmd) CommandFactory.createCommand(SelectContractPricesCmd.NAME, storeId);

		selectPricesCmd.setItemPriceInfo(iItemPriceInfo);
		selectPricesCmd.setCurrency(requestedCurrency);	
		selectPricesCmd.setCommandContext(commandContext);
		selectPricesCmd.setErrorMode(iErrorMode);
		selectPricesCmd.setStoreId(getStoreAB().getStoreEntityIdInEJBType());
		selectPricesCmd.execute();
		iItemPriceInfo = selectPricesCmd.getItemPriceInfo();

		inTradingIdOut	= iItemPriceInfo[0].getTradingId();
		inTcIdOut		= iItemPriceInfo[0].getTcId();
		inOfferIdOut	= iItemPriceInfo[0].getOfferId();
		iOfferOut 		= iItemPriceInfo[0].getOffer();
		// PriceList: for roundCustomized
		iPrice			= PriceCalculationHelper.getInstance().setRoundCustomizedPrice(iItemPriceInfo[0].getUnitPrice(), getStoreAB());
		iBasePrice 		= iItemPriceInfo[0].getBasePrice();

		ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, SELECT_PRICES, "selected trading id: " + inTradingIdOut + 
				" tc id: " + inTcIdOut + " offer id: " + inOfferIdOut + " unit price: " + iPrice);
	
		if(iCatEntryPrices == null || iCatEntryPrices.length <= 0) {
			iCatEntryPrices = new CatEntryPrices[iItemPriceInfo.length];
		}	
	
		for(int i=0; i<iItemPriceInfo.length; i++) {

			if(iCatEntryPrices[i] == null) {
				iCatEntryPrices[i] = new CatEntryPrices(iItemPriceInfo[i].getCatalogEntryId(), iItemPriceInfo[i].getQuantity());
			}	
					
			iCatEntryPrices[i].setTradingId(iItemPriceInfo[i].getTradingId());
			iCatEntryPrices[i].setTcId(iItemPriceInfo[i].getTcId());
			iCatEntryPrices[i].setOfferId(iItemPriceInfo[i].getOfferId());
			iCatEntryPrices[i].setOffer(iItemPriceInfo[i].getOffer());
			iCatEntryPrices[i].setUnitPrice(PriceCalculationHelper.getInstance().setRoundCustomizedPrice(iItemPriceInfo[i].getUnitPrice(), getStoreAB()));
			iCatEntryPrices[i].setBasePrice(iItemPriceInfo[i].getBasePrice());

			Long[] allTradingIds = iItemPriceInfo[i].getTradingIds();	
			iCatEntryPrices[i].setAllTradingIds(allTradingIds);
			iCatEntryPrices[i].initOutput(allTradingIds.length);
		
			iCatEntryPrices[i].setTradingTcIds(iItemPriceInfo[i].getTradingTcIds());
			iCatEntryPrices[i].setTradingOfferIds(iItemPriceInfo[i].getTradingOfferIds());
			iCatEntryPrices[i].setTradingOffers(iItemPriceInfo[i].getTradingOffers());
			iCatEntryPrices[i].setTradingUnitPrices(iItemPriceInfo[i].getTradingUnitPrices(), getStoreAB());
			iCatEntryPrices[i].setTradingBasePrices(iItemPriceInfo[i].getTradingBasePrices());	
				
		}
		
	} catch (javax.ejb.CreateException ex) {
		throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, SELECT_PRICES, ex);
	} catch (javax.ejb.FinderException ex) {
		throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, SELECT_PRICES, ex);
	} catch (javax.naming.NamingException ex) {
		throw new ECSystemException(ECMessage._ERR_GENERIC, CLASS_NAME, SELECT_PRICES, ex);
	} catch (java.rmi.RemoteException ex) {
		throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, SELECT_PRICES, ex);
	}		

	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, SELECT_PRICES);
		
}
/**
 * Sets the base price for the input catentry.
 * 
 * @param aBasePrice the price (and currency) found for the input catalog entry 
 * before any conversion. This is only set when the requested currency is different 
 * from the base currency and the base price must be converted to the requested price.
 * property. 
 */
private void setBasePrice(MonetaryAmount aBasePrice) {
	iBasePrice = aBasePrice;
}
/**
 * Sets the id of the input catalog entry.
 */
public void setCatEntryId(Long anCatEntryId) {
	inCatEntryId = anCatEntryId;
}
/**
 * Sets the information of one or more catalog entries (Future Support).
 */
public void setCatEntryPrices(CatEntryPrices[] aCatEntryPrices) {
	iCatEntryPrices = aCatEntryPrices;
}
/**
 * Sets the currency to be used for finding the price (Optional). 
 */
public void setCurrency(String astrCurrency) {
	istrCurrency = astrCurrency;
}
/**
 * Sets the flag to indicate if exception will be thrown when the price cannot be found 
 * for the input catentry (Optional).
 */
public void setErrorMode(boolean aErrorMode) {
	iErrorMode = aErrorMode;
}
/**
 * Sets the item info structure for internal use on calculating the prices.
 *
 * @param aItemPriceInfo the item info structure for storing information 
 * such as the trading agreements, price lists that will be used for
 * calculating the prices of the items. 
 */
private void setItemPriceInfo(ItemPriceInfo[] aItemPriceInfo) {
	iItemPriceInfo = aItemPriceInfo;
}
/**
 * Sets the member id to be used for verifying and searching the trading agreements (Optional).
 */
public void setMemberId(Long anMemberId) {
	inMemberId = anMemberId;
}
/**
 * Sets the output offer Access Bean.
 *
 * @param aOfferOut the access bean of an offer that was used to  
 * determine the price.
 */
private void setOffer(OfferAccessBean aOfferOut) {
	iOfferOut = aOfferOut;
}
/**
 * Sets the id of the output offer id.
 *
 * @param anOfferIdOut the reference number of an offer that was used to  
 * determine the price.
 */
private void setOfferId(Long anOfferIdOut) {
	inOfferIdOut = anOfferIdOut;
}
/**
 * Sets the ids of the input offers to be used for price (Optional).
 */
public void setOfferIds(Long[] aOfferIds) {
	iOfferIdsIn = aOfferIds;
}
/**
 * Sets the orderitem Access Bean.
 *
 * @param aOrderItemAB an order item access bean. 
 */
public void setOrderItemAccessBean(OrderItemAccessBean aOrderItemAB) {
	iOrderItemAB = aOrderItemAB;
}
/**
 * Sets the quantity amount of the input catalog entry that is to be priced (Optional).
 */
public void setQuantity(QuantityAmount aQuantityAmount) {
	iQuantityAmount = aQuantityAmount;
}
/**
 * Sets the store Access Bean.
 *
 * @param aStoreAB the access bean of the store. 
 */
private void setStoreAB(StoreAccessBean aStoreAB) {
	iStoreAB = aStoreAB;
}
/**
 * Sets the id of the input store to be used for calculating the price (Optional). 
 */
public void setStoreId(java.lang.Integer aStoreId) {
	inStoreId = aStoreId;
}
/**
 * Sets the ids of the input trading agreements (Optional).
 */
public void setTradingIds(Long[] aTradingIds) {
	iTradingIdsIn = aTradingIds;
}
/**
 * Checks mandatory parameters.
 */
public void validateParameters() throws ECException {
	
	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CHECK_PARAMETERS);

	// check the store object - ensure that the storeId is a valid store
	if (getStoreAB() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, CHECK_PARAMETERS , new Object[] {ECConstants.EC_STORE_ID});
	}

	// PriceList: check the catalog entry Id or other related information
	if (getCatEntryId() == null && 
		(getCatEntryPrices() == null || getCatEntryPrices().length <= 0)) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, CHECK_PARAMETERS, new Object[] {OrderConstants.EC_CATALOG_ENTRY_ID});
	}

	// check the currency
	if (getCurrency() == null) {
		throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER, CLASS_NAME, CHECK_PARAMETERS, new Object[] {ECConstants.EC_CURRENCY_ID});
	}
		
	ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, CHECK_PARAMETERS);
}
}
